import os
from typing import Literal, Optional

from zai import ZhipuAiClient

from powermem.integrations.embeddings.base import EmbeddingBase
from powermem.integrations.embeddings.config.base import BaseEmbedderConfig


class ZaiEmbedding(EmbeddingBase):
    """
    Zhipu AI (Z.ai) Embedding implementation.
    
    Reference: https://docs.bigmodel.cn/cn/guide/develop/python/introduction
    
    Supported models:
    - embedding-3: Latest embedding model
    - embedding-2: Standard embedding model
    """

    def __init__(self, config: Optional[BaseEmbedderConfig] = None):
        super().__init__(config)

        # Set default model and embedding dimensions
        self.config.model = self.config.model or "embedding-3"
        self.config.embedding_dims = self.config.embedding_dims or 2048

        # Get API key from config or environment
        api_key = self.config.api_key or os.getenv("ZAI_API_KEY")

        # Initialize Zhipu AI client
        self.client = ZhipuAiClient(api_key=api_key)

    def embed(self, text, memory_action: Optional[Literal["add", "search", "update"]] = None):
        """
        Get the embedding for the given text using Zhipu AI.

        Args:
            text (str): The text to embed.
            memory_action (optional): The type of embedding to use. Must be one of "add", "search", or "update". Defaults to None.
        Returns:
            list: The embedding vector.
        """
        text = text.replace("\n", " ")
        response = self.client.embeddings.create(
            input=[text],
            model=self.config.model
        )
        return response.data[0].embedding



from typing import Optional

from pydantic import AliasChoices, Field

from powermem.integrations.embeddings.config.sparse_base import BaseSparseEmbedderConfig
from powermem.settings import settings_config


class QwenSparseEmbeddingConfig(BaseSparseEmbedderConfig):
    _provider_name = "qwen"
    _class_path = "powermem.integrations.embeddings.qwen_sparse.QwenSparseEmbedding"

    model_config = settings_config("SPARSE_EMBEDDER_", extra="forbid", env_file=None)

    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "SPARSE_EMBEDDER_API_KEY",
            "DASHSCOPE_API_KEY",
        ),
    )
    base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "base_url",
            "SPARSE_EMBEDDING_BASE_URL",
            "DASHSCOPE_BASE_URL",
        ),
    )
    model: Optional[str] = Field(default=None)
    embedding_dims: Optional[int] = Field(default=None)



from typing import Any, ClassVar, Dict, Optional

from pydantic import AliasChoices, Field, model_validator
from powermem.settings import settings_config

from powermem.storage.config.base import BaseVectorStoreConfig, BaseGraphStoreConfig


class OceanBaseConfig(BaseVectorStoreConfig):
    _provider_name = "oceanbase"
    _class_path = "powermem.storage.oceanbase.oceanbase.OceanBaseVectorStore"
    
    try:
        from pyobvector import ObVecClient
    except ImportError:
        raise ImportError("The 'pyobvector' library is required. Please install it using 'pip install pyobvector'.")
    ObVecClient: ClassVar[type] = ObVecClient

    model_config = settings_config("VECTOR_STORE_", extra="forbid", env_file=None)

    collection_name: str = Field(
        default="power_mem",
        validation_alias=AliasChoices(
            "collection_name",
            "VECTOR_STORE_COLLECTION_NAME",
            "OCEANBASE_COLLECTION",
        ),
        description="Default name for the collection"
    )

    # Connection parameters
    host: str = Field(
        default="127.0.0.1",
        validation_alias=AliasChoices(
            "host",
            "OCEANBASE_HOST",
        ),
        description="OceanBase server host"
    )
    
    port: str = Field(
        default="2881",
        validation_alias=AliasChoices(
            "port",
            "OCEANBASE_PORT",
        ),
        description="OceanBase server port"
    )
    
    user: str = Field(
        default="root@test",
        validation_alias=AliasChoices(
            "OCEANBASE_USER",
            "user", # avoid using system USER environment variable first
        ),
        description="OceanBase username"
    )
    
    password: str = Field(
        default="",
        validation_alias=AliasChoices(
            "password",
            "OCEANBASE_PASSWORD",
        ),
        description="OceanBase password"
    )
    
    db_name: str = Field(
        default="test",
        validation_alias=AliasChoices(
            "db_name",
            "OCEANBASE_DATABASE",
        ),
        description="OceanBase database name"
    )

    connection_args: Optional[dict] = Field(
        default=None,
        validation_alias=AliasChoices(
            "connection_args",
        ),
        description="OceanBase connection args"
    )

    # Vector index parameters
    index_type: str = Field(
        default="HNSW",
        validation_alias=AliasChoices(
            "index_type",
            "OCEANBASE_INDEX_TYPE",
        ),
        description="Type of vector index (HNSW, IVF, FLAT, etc.)"
    )
    
    vidx_metric_type: str = Field(
        default="l2",
        validation_alias=AliasChoices(
            "vidx_metric_type",
            "OCEANBASE_VECTOR_METRIC_TYPE",
        ),
        description="Distance metric (l2, inner_product, cosine)"
    )
    
    embedding_model_dims: Optional[int] = Field(
        default=None,
        validation_alias=AliasChoices(
            "embedding_model_dims",
            "OCEANBASE_EMBEDDING_MODEL_DIMS",
        ),
        description="Dimension of vectors"
    )

    # Advanced parameters
    vidx_algo_params: Optional[Dict[str, Any]] = Field(
        default=None,
        description="Index algorithm parameters"
    )
    
    normalize: bool = Field(
        default=False,
        description="Whether to normalize vectors"
    )
    
    include_sparse: bool = Field(
        default=False,
        validation_alias=AliasChoices(
            "include_sparse",
            "OCEANBASE_INCLUDE_SPARSE",
            "SPARSE_VECTOR_ENABLE",
        ),
        description="Whether to include sparse vector support"
    )
    
    hybrid_search: bool = Field(
        default=True,
        validation_alias=AliasChoices(
            "hybrid_search",
        ),
        description="Whether to enable hybrid search"
    )

    enable_native_hybrid: bool = Field(
        default=False,
        validation_alias=AliasChoices(
            "enable_native_hybrid",
            "OCEANBASE_ENABLE_NATIVE_HYBRID",
        ),
        description="Whether to enable OceanBase native hybrid search"
    )
    
    auto_configure_vector_index: bool = Field(
        default=True,
        description="Whether to automatically configure vector index settings"
    )

    # Fulltext search parameters
    fulltext_parser: str = Field(
        default="ik",
        description="Fulltext parser type (ik, ngram, ngram2, beng, space)"
    )

    # Field names
    primary_field: str = Field(
        default="id",
        validation_alias=AliasChoices(
            "primary_field",
            "OCEANBASE_PRIMARY_FIELD",
        ),
        description="Primary key field name"
    )
    
    vector_field: str = Field(
        default="embedding",
        validation_alias=AliasChoices(
            "vector_field",
            "OCEANBASE_VECTOR_FIELD",
        ),
        description="Vector field name"
    )
    
    text_field: str = Field(
        default="document",
        validation_alias=AliasChoices(
            "text_field",
            "OCEANBASE_TEXT_FIELD",
        ),
        description="Text field name"
    )
    
    metadata_field: str = Field(
        default="metadata",
        validation_alias=AliasChoices(
            "metadata_field",
            "OCEANBASE_METADATA_FIELD",
        ),
        description="Metadata field name"
    )
    
    vidx_name: str = Field(
        default="vidx",
        validation_alias=AliasChoices(
            "vidx_name",
            "OCEANBASE_VIDX_NAME",
        ),
        description="Vector index name"
    )

    vector_weight: float = Field(
        default=0.5,
        description="Weight for vector search"
    )
    
    fts_weight: float = Field(
        default=0.5,
        description="Weight for fulltext search"
    )
    
    sparse_weight: Optional[float] = Field(
        default=None,
        description="Weight for sparse vector search"
    )



class OceanBaseGraphConfig(BaseGraphStoreConfig):
    """Configuration for OceanBase graph store."""
    
    _provider_name = "oceanbase"
    _class_path = "powermem.storage.oceanbase.oceanbase_graph.MemoryGraph"
    
    model_config = settings_config("GRAPH_STORE_", extra="forbid", env_file=None)
    
    # All fields (connection, vector, max_hops) are inherited from BaseGraphStoreConfig
    # No additional fields needed for OceanBase GraphStore at this time



from typing import Optional

from pydantic import AliasChoices, Field

from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.settings import settings_config


class QwenASRConfig(BaseLLMConfig):
    """
    Configuration class for Qwen ASR-specific parameters.
    Inherits from BaseLLMConfig and adds ASR-specific settings.
    """

    _provider_name = "qwen_asr"
    _class_path = "powermem.integrations.llm.qwen_asr.QwenASR"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)

    # Override base fields with ASR-specific validation_alias
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "LLM_API_KEY",
            "QWEN_API_KEY",
            "DASHSCOPE_API_KEY",
        ),
        description="DashScope API key for Qwen ASR"
    )

    # ASR-specific fields
    dashscope_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "dashscope_base_url",
            "QWEN_LLM_BASE_URL",
        ),
        description="DashScope API base URL"
    )
    
    asr_options: Optional[dict] = Field(
        default_factory=lambda: {"enable_itn": True},
        description="ASR-specific options (e.g., language, enable_itn)"
    )
    
    result_format: str = Field(
        default="message",
        description="Result format for ASR response"
    )



"""
PowerMem Script Package

Provides script management tools for database upgrades, migrations, and maintenance.

Usage:
    from script import ScriptManager
    from powermem import auto_config, Memory
    
    # List available scripts
    ScriptManager.list_scripts()
    
    # View script details
    ScriptManager.info('upgrade-sparse-vector')
    
    # Execute upgrade script
    config = auto_config()
    ScriptManager.run('upgrade-sparse-vector', config)
    
    # Execute migration script
    memory = Memory(config=config)
    ScriptManager.run('migrate-sparse-vector', memory, batch_size=1000, workers=3)
"""

from script.script_manager import ScriptManager

__all__ = ['ScriptManager']



"""
Storage factory for creating storage instances

This module provides a factory for creating different storage backends.
"""

import importlib

# Import all provider configs to trigger auto-registration
from powermem.storage.config.base import BaseVectorStoreConfig, BaseGraphStoreConfig
from powermem.storage.config.oceanbase import OceanBaseConfig, OceanBaseGraphConfig
from powermem.storage.config.pgvector import PGVectorConfig
from powermem.storage.config.sqlite import SQLiteConfig


def load_class(class_type):
    module_path, class_name = class_type.rsplit(".", 1)
    module = importlib.import_module(module_path)
    return getattr(module, class_name)

class VectorStoreFactory:
    @classmethod
    def create(cls, provider_name, config):
        """
        Create a VectorStore instance with the appropriate configuration.
        
        Args:
            provider_name (str): The provider name (e.g., 'oceanbase', 'pgvector', 'sqlite')
            config: Configuration object or dict. If dict, will convert to provider config
        
        Returns:
            Configured VectorStore instance
        
        Raises:
            ValueError: If provider is not supported
        """
        # Handle postgres alias
        if provider_name == "postgres":
            provider_name = "pgvector"
        
        # 1. Get class_path from registry
        class_path = BaseVectorStoreConfig.get_provider_class_path(provider_name)
        if not class_path:
            raise ValueError(f"Unsupported VectorStore provider: {provider_name}")
        
        # 2. Get config_cls from registry
        config_cls = BaseVectorStoreConfig.get_provider_config_cls(provider_name) or BaseVectorStoreConfig
        
        # 3. Handle config parameter
        if isinstance(config, dict):
            # Convert dict to provider config instance
            provider_config = config_cls(**config)
        elif isinstance(config, BaseVectorStoreConfig):
            # Use config instance directly
            provider_config = config
        else:
            raise TypeError(f"config must be BaseVectorStoreConfig or dict, got {type(config)}")
        
        # 4. Export to dict for VectorStore constructor
        config_dict = provider_config.model_dump(exclude_none=True)
        
        # 5. Create VectorStore instance
        vector_store_class = load_class(class_path)
        return vector_store_class(**config_dict)

    @classmethod
    def register_provider(cls, name: str, class_path: str, config_class=None):
        """
        Register a new vector store provider.
        
        Args:
            name (str): Provider name
            class_path (str): Full path to VectorStore class
            config_class: Configuration class for the provider (defaults to BaseVectorStoreConfig)
        """
        if config_class is None:
            config_class = BaseVectorStoreConfig
        
        # Register directly in BaseVectorStoreConfig registry
        BaseVectorStoreConfig._registry[name] = config_class
        BaseVectorStoreConfig._class_paths[name] = class_path

    @classmethod
    def get_supported_providers(cls) -> list:
        """
        Get list of supported providers.
        
        Returns:
            list: List of supported provider names
        """
        return list(BaseVectorStoreConfig._registry.keys())

    @classmethod
    def reset(cls, instance):
        instance.reset()
        return instance


class GraphStoreFactory:
    """
    Factory for creating MemoryGraph instances for different graph store providers.
    Usage: GraphStoreFactory.create(provider_name, config)
    """

    @classmethod
    def create(cls, provider_name, config):
        """
        Create a GraphStore instance with the appropriate configuration.
        
        Args:
            provider_name (str): The provider name (e.g., 'oceanbase')
            config: Configuration object or dict. If dict, will convert to provider config
        
        Returns:
            Configured GraphStore instance
        
        Raises:
            ValueError: If provider is not supported
        """
        # 1. Get class_path from registry
        class_path = BaseGraphStoreConfig.get_provider_class_path(provider_name)
        if not class_path:
            raise ValueError(f"Unsupported GraphStore provider: {provider_name}")
        
        # 2. Get config_cls from registry
        config_cls = BaseGraphStoreConfig.get_provider_config_cls(provider_name) or BaseGraphStoreConfig
        
        # 3. Handle config parameter
        if isinstance(config, dict):
            # Convert dict to provider config instance
            provider_config = config_cls(**config)
        elif isinstance(config, BaseGraphStoreConfig):
            # Use config instance directly
            provider_config = config
        else:
            raise TypeError(f"config must be BaseGraphStoreConfig or dict, got {type(config)}")
        
        # 4. Export to dict for GraphStore constructor
        config_dict = provider_config.model_dump(exclude_none=True)
        
        # 5. Create GraphStore instance
        graph_store_class = load_class(class_path)
        return graph_store_class(config_dict)

    @classmethod
    def register_provider(cls, name: str, class_path: str, config_class=None):
        """
        Register a new graph store provider.
        
        Args:
            name (str): Provider name
            class_path (str): Full path to GraphStore class
            config_class: Configuration class for the provider (defaults to BaseGraphStoreConfig)
        """
        if config_class is None:
            config_class = BaseGraphStoreConfig
        
        # Register directly in BaseGraphStoreConfig registry
        BaseGraphStoreConfig._registry[name] = config_class
        BaseGraphStoreConfig._class_paths[name] = class_path

    @classmethod
    def get_supported_providers(cls) -> list:
        """
        Get list of supported providers.
        
        Returns:
            list: List of supported provider names
        """
        return list(BaseGraphStoreConfig._registry.keys())




"""
Rerank factory for creating rerank instances

This module provides a factory for creating different rerank instances.
"""

import importlib
from typing import Optional, Union

from powermem.integrations.rerank.config.base import BaseRerankConfig
# Trigger automatic registration by importing provider configs
from powermem.integrations.rerank.config.providers import (
    QwenRerankConfig,
    JinaRerankConfig,
    ZaiRerankConfig,
    GenericRerankConfig,
)


class RerankFactory:
    """Factory class for creating rerank model instances"""

    @classmethod
    def create(
        cls,
        provider_name: str = "qwen",
        config: Optional[Union[dict, BaseRerankConfig]] = None
    ):
        """
        Create a rerank instance based on provider name.

        Args:
            provider_name (str): Name of the rerank provider. Defaults to "qwen"
            config (Optional[Union[dict, BaseRerankConfig]]): 
                Configuration dictionary or BaseRerankConfig instance for the rerank model

        Returns:
            RerankBase: An instance of the requested rerank provider

        Raises:
            ValueError: If the provider is not supported
            TypeError: If config type is invalid
        """
        # Get config class from registry
        config_cls = BaseRerankConfig.get_provider_config_cls(provider_name)
        if not config_cls:
            supported = ", ".join(BaseRerankConfig._registry.keys())
            raise ValueError(
                f"Unsupported rerank provider: {provider_name}. "
                f"Supported providers: {supported}"
            )

        # Get class path from registry
        class_path = BaseRerankConfig.get_provider_class_path(provider_name)
        if not class_path:
            raise ValueError(f"No class path registered for provider: {provider_name}")

        # Load reranker class
        module_path, class_name = class_path.rsplit(".", 1)
        module = importlib.import_module(module_path)
        reranker_class = getattr(module, class_name)

        # Create config instance
        if config is None:
            # Use default config
            config_instance = config_cls()
        elif isinstance(config, dict):
            # Create config from dict
            config_instance = config_cls(**config)
        elif isinstance(config, BaseRerankConfig):
            # Use provided config instance directly
            config_instance = config
        else:
            raise TypeError(
                f"config must be dict or BaseRerankConfig, got {type(config)}"
            )

        # Create and return reranker instance
        return reranker_class(config_instance)

    @classmethod
    def list_providers(cls) -> list:
        """
        List all supported rerank providers.

        Returns:
            list: List of supported provider names
        """
        return list(BaseRerankConfig._registry.keys())




"""
PowerMem Script Manager

Provides Python API to manage and execute various maintenance and upgrade scripts.

Usage example:
    from powermem import auto_config, Memory
    from script import ScriptManager
    
    # List available scripts
    ScriptManager.list_scripts()
    
    # View script details
    ScriptManager.info('migrate-sparse-vector')
    
    # Execute upgrade script
    config = auto_config()
    ScriptManager.run('upgrade-sparse-vector', config)
    
    # Execute migration script
    memory = Memory(config=config)
    ScriptManager.run('migrate-sparse-vector', memory, batch_size=1000)
"""

import importlib
import inspect
import json
import logging
from pathlib import Path
from typing import Any, Dict, Optional

logger = logging.getLogger(__name__)


class ScriptManager:
    """Script Manager - Used to execute maintenance and upgrade scripts"""
    
    @classmethod
    def _get_config_path(cls) -> Path:
        """Get the path to scripts_config.json relative to this module"""
        return Path(__file__).parent / "scripts_config.json"
    
    @classmethod
    def _load_config(cls) -> Dict[str, Any]:
        """Load script configuration"""
        config_path = cls._get_config_path()
        
        if not config_path.exists():
            raise FileNotFoundError(
                f"Configuration file not found: {config_path}\n"
                f"Please ensure scripts_config.json exists in the script package"
            )
        
        try:
            with open(config_path, 'r', encoding='utf-8') as f:
                config = json.load(f)
            return config
        except Exception as e:
            raise RuntimeError(f"Failed to load configuration file: {e}")
    
    @classmethod
    def _get_script_info(cls, script_name: str, config: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Get script information
        
        Args:
            script_name: Script name
            config: Optional config dict, if not provided will load from default path
            
        Returns:
            Script configuration information
            
        Raises:
            ValueError: If script does not exist
        """
        if config is None:
            config = cls._load_config()
        scripts = config.get('scripts', {})
        if script_name not in scripts:
            available = ', '.join(scripts.keys())
            raise ValueError(
                f"Unknown script: {script_name}\n"
                f"Available scripts: {available}\n"
                f"Use ScriptManager.list_scripts() to view details"
            )
        return scripts[script_name]
    
    @classmethod
    def list_scripts(cls, category: Optional[str] = None, verbose: bool = True) -> Dict[str, Dict]:
        """
        List all available scripts
        
        Args:
            category: Optional, only list scripts in the specified category
            verbose: Whether to print to console (default True)
            
        Returns:
            Script dictionary, key is script name, value is script info
        """
        config = cls._load_config()
        scripts = config.get('scripts', {})
        categories = config.get('categories', {})
        
        # Filter by category
        filtered_scripts = {}
        for script_name, script_info in scripts.items():
            if category and script_info.get('category') != category:
                continue
            filtered_scripts[script_name] = script_info
        
        if verbose:
            cls._print_scripts(filtered_scripts, categories)
        
        return filtered_scripts
    
    @classmethod
    def _print_scripts(cls, scripts: Dict[str, Dict], categories: Dict[str, str]) -> None:
        """Print script list"""
        logger.info("\n" + "=" * 70)
        logger.info("PowerMem Available Scripts")
        logger.info("=" * 70)
        
        # Organize scripts by category
        scripts_by_category: Dict[str, list] = {}
        for script_name, script_info in scripts.items():
            script_category = script_info.get('category', 'uncategorized')
            if script_category not in scripts_by_category:
                scripts_by_category[script_category] = []
            scripts_by_category[script_category].append((script_name, script_info))
        
        # Print scripts for each category
        for cat, script_list in sorted(scripts_by_category.items()):
            category_desc = categories.get(cat, cat)
            logger.info(f"\n【{category_desc}】")
            logger.info("-" * 70)
            
            for script_name, script_info in sorted(script_list):
                desc = script_info.get('description', 'No description')
                destructive = script_info.get('destructive', False)
                warning = " ⚠️  Destructive Operation" if destructive else ""
                
                # Try to get the type hint of the first parameter
                param_hint = cls._get_first_param_hint(script_name, script_info)
                param_info = f" (requires: {param_hint})" if param_hint else ""
                
                logger.info(f"  • {script_name}{warning}")
                logger.info(f"    {desc}{param_info}")
        
        logger.info("\n" + "=" * 70)
        logger.info("Usage: ScriptManager.run('script_name', param)")
        logger.info("Tip: Use ScriptManager.info('script_name') to view detailed parameters")
        logger.info("=" * 70 + "\n")
    
    @classmethod
    def _get_first_param_hint(cls, script_name: str, script_info: Dict[str, Any]) -> Optional[str]:
        """Get the type hint of the first parameter of the script"""
        try:
            module = importlib.import_module(script_info['module'])
            func = getattr(module, script_info['function'])
            sig = inspect.signature(func)
            
            # Get the first parameter
            params = list(sig.parameters.values())
            if params:
                first_param = params[0]
                if first_param.annotation != inspect.Parameter.empty:
                    annotation = first_param.annotation
                    # Handle string type annotations (e.g., 'Memory')
                    if isinstance(annotation, str):
                        return annotation
                    # Handle type objects
                    return getattr(annotation, '__name__', str(annotation))
        except Exception:
            pass
        return None
    
    @classmethod
    def info(cls, script_name: str) -> None:
        """
        Display detailed information about the script, including parameter signatures and documentation
        
        Args:
            script_name: Script name
            
        Example:
            ScriptManager.info('migrate-sparse-vector')
        """
        try:
            # Get script configuration information
            script_info = cls._get_script_info(script_name)
            
            # Load module and function
            module = importlib.import_module(script_info['module'])
            func = getattr(module, script_info['function'])
            
            # Get function signature
            sig = inspect.signature(func)
            
            # Get function documentation
            doc = inspect.getdoc(func) or "No documentation available"
            
            # Print information
            logger.info("\n" + "=" * 70)
            logger.info(f"Script: {script_name}")
            logger.info("=" * 70)
            logger.info(f"Category: {script_info.get('category', 'N/A')}")
            logger.info(f"Description: {script_info.get('description', 'N/A')}")
            
            if script_info.get('destructive', False):
                logger.info("⚠️  Warning: This is a DESTRUCTIVE operation!")
            
            logger.info("\n" + "-" * 70)
            logger.info("Parameters:")
            logger.info("-" * 70)
            
            # Parse parameter information
            for param_name, param in sig.parameters.items():
                # Parameter type
                param_type = "Any"
                if param.annotation != inspect.Parameter.empty:
                    annotation = param.annotation
                    if isinstance(annotation, str):
                        param_type = annotation
                    else:
                        param_type = getattr(annotation, '__name__', str(annotation))
                
                # Default value
                if param.default != inspect.Parameter.empty:
                    default_str = f", default={param.default}"
                else:
                    default_str = " (required)"
                
                logger.info(f"  {param_name} ({param_type}){default_str}")
            
            logger.info("\n" + "-" * 70)
            logger.info("Documentation:")
            logger.info("-" * 70)
            for line in doc.split('\n'):
                logger.info(f"  {line}")
            
            logger.info("\n" + "=" * 70)
            logger.info("Usage Example:")
            logger.info("=" * 70)
            
            # Generate usage example based on the first parameter type
            params = list(sig.parameters.values())
            if params:
                first_param = params[0]
                param_type_hint = None
                if first_param.annotation != inspect.Parameter.empty:
                    annotation = first_param.annotation
                    param_type_hint = annotation if isinstance(annotation, str) else getattr(annotation, '__name__', None)
                
                if param_type_hint and 'Memory' in param_type_hint:
                    logger.info(f"  from powermem import Memory, auto_config")
                    logger.info(f"  from script import ScriptManager")
                    logger.info(f"  ")
                    logger.info(f"  config = auto_config()")
                    logger.info(f"  memory = Memory(config=config)")
                    logger.info(f"  ScriptManager.run('{script_name}', memory)")
                else:
                    logger.info(f"  from powermem import auto_config")
                    logger.info(f"  from script import ScriptManager")
                    logger.info(f"  ")
                    logger.info(f"  config = auto_config()")
                    logger.info(f"  ScriptManager.run('{script_name}', config)")
            
            logger.info("=" * 70 + "\n")
            
        except Exception as e:
            logger.error(f"Failed to get script info: {e}", exc_info=True)
    
    @classmethod
    def run(cls, script_name: str, param: Any, **kwargs) -> bool:
        """
        Execute the specified script
        
        Args:
            script_name: Script name
            param: Script parameter (config dict/MemoryConfig for upgrade/downgrade scripts, 
                   Memory instance for migration scripts)
            **kwargs: Additional parameters to pass to the script function
            
        Returns:
            bool: Returns True on success, False on failure
            
        Raises:
            ValueError: If param is None or script does not exist
            
        Examples:
            # Upgrade/downgrade scripts (use config)
            config = auto_config()
            ScriptManager.run('upgrade-sparse-vector', config)
            
            # Migration scripts (use Memory instance)
            memory = Memory(config=config)
            ScriptManager.run('migrate-sparse-vector', memory, batch_size=1000)
        """
        if param is None:
            raise ValueError(
                "param parameter is required\n"
                "Example:\n"
                "  from powermem import auto_config, Memory\n"
                "  from script import ScriptManager\n"
                "  config = auto_config()\n"
                "  # For upgrade/downgrade:\n"
                "  ScriptManager.run('upgrade-sparse-vector', config)\n"
                "  # For migration:\n"
                "  memory = Memory(config=config)\n"
                "  ScriptManager.run('migrate-sparse-vector', memory)"
            )
        
        try:
            # Load scripts configuration (not to be confused with memory config parameter)
            scripts_config = cls._load_config()
            script_info = cls._get_script_info(script_name, scripts_config)
            
            # Display script information
            logger.info(f"\nPreparing to execute script: {script_name}")
            logger.info(f"Description: {script_info.get('description', 'No description')}")
            
            # Check if it's a destructive operation
            if script_info.get('destructive', False):
                logger.warning("\n⚠️  Warning: This is a destructive operation that may delete data!")
                confirm = input("Confirm to continue? (yes/no): ").strip().lower()
                if confirm not in ['yes', 'y']:
                    logger.info("Operation cancelled")
                    return False
            
            # Load script module and function
            module_name = script_info['module']
            function_name = script_info['function']
            
            logger.info(f"Loading module: {module_name}")
            module = importlib.import_module(module_name)
            script_func = getattr(module, function_name)
            
            # Execute script
            logger.info(f"Executing script function: {function_name}")
            result = script_func(param, **kwargs)
            
            if result:
                logger.info(f"\n✓ Script '{script_name}' executed successfully!")
            else:
                logger.error(f"\n✗ Script '{script_name}' execution failed")
            
            return result
            
        except TypeError as e:
            # Catch parameter errors and provide friendly hints
            error_msg = str(e)
            logger.error(f"\n✗ Parameter Error: {error_msg}")
            
            try:
                sig = inspect.signature(script_func)
                logger.error(f"\nExpected function signature:")
                logger.error(f"  {function_name}{sig}")
                logger.error(f"\nFor detailed parameter information, run:")
                logger.error(f"  ScriptManager.info('{script_name}')")
            except Exception:
                pass
            
            return False
            
        except Exception as e:
            logger.error(f"Error occurred while executing script: {e}", exc_info=True)
            logger.error(f"\n✗ Error: {e}")
            return False



from typing import Any, ClassVar, Dict, Optional

from pydantic import Field
from pydantic_settings import BaseSettings

from powermem.settings import settings_config


class BaseEmbedderConfig(BaseSettings):
    """Common embedding configuration shared by all providers."""

    model_config = settings_config("EMBEDDING_", extra="allow", env_file=None)

    _provider_name: ClassVar[Optional[str]] = None
    _class_path: ClassVar[Optional[str]] = None
    _registry: ClassVar[dict[str, type["BaseEmbedderConfig"]]] = {}
    _class_paths: ClassVar[dict[str, str]] = {}

    @classmethod
    def _register_provider(cls) -> None:
        provider = getattr(cls, "_provider_name", None)
        class_path = getattr(cls, "_class_path", None)
        if provider:
            BaseEmbedderConfig._registry[provider] = cls
            if class_path:
                BaseEmbedderConfig._class_paths[provider] = class_path

    def __init_subclass__(cls, **kwargs) -> None:
        super().__init_subclass__(**kwargs)
        cls._register_provider()

    @classmethod
    def __pydantic_init_subclass__(cls, **kwargs) -> None:
        super().__pydantic_init_subclass__(**kwargs)
        cls._register_provider()

    @classmethod
    def get_provider_config_cls(cls, provider: str) -> Optional[type["BaseEmbedderConfig"]]:
        return cls._registry.get(provider)

    @classmethod
    def get_provider_class_path(cls, provider: str) -> Optional[str]:
        return cls._class_paths.get(provider)

    @classmethod
    def has_provider(cls, provider: str) -> bool:
        return provider in cls._registry

    model: Optional[Any] = Field(
        default=None,
        description="Embedding model name or provider-specific model object.",
    )
    api_key: Optional[str] = Field(
        default=None,
        description="API key used for provider authentication.",
    )
    embedding_dims: Optional[int] = Field(
        default=None,
        description="Embedding vector dimensions, when configurable by provider.",
    )

    def to_component_dict(self) -> Dict[str, Any]:
        return {
            "provider": self._provider_name,
            "config": self.model_dump(exclude_none=True)
        }



from typing import Any, ClassVar, Dict, Optional, Union
from pydantic import AliasChoices, Field
from pydantic_settings import BaseSettings
from powermem.settings import settings_config


class BaseVectorStoreConfig(BaseSettings):
    """
    Base configuration class for all vector store providers.
    
    This class provides common fields and validation logic shared
    across all vector store implementations.
    """
    
    # Model config
    model_config = settings_config("VECTOR_STORE_", extra="allow", env_file=None)
    
    # Registry mechanism (same as LLM/Rerank)
    _provider_name: ClassVar[Optional[str]] = None
    _class_path: ClassVar[Optional[str]] = None
    _registry: ClassVar[dict[str, type["BaseVectorStoreConfig"]]] = {}
    _class_paths: ClassVar[dict[str, str]] = {}
    
    # Common fields across all providers
    collection_name: str = Field(
        default="memories",
        description="Name of the collection/table"
    )
    
    embedding_model_dims: Optional[int] = Field(
        default=None,
        description="Dimension of embedding vectors"
    )
    
    @classmethod
    def _register_provider(cls) -> None:
        """Register provider in the global registry."""
        provider = getattr(cls, "_provider_name", None)
        class_path = getattr(cls, "_class_path", None)
        if provider:
            BaseVectorStoreConfig._registry[provider] = cls
            if class_path:
                BaseVectorStoreConfig._class_paths[provider] = class_path
    
    def __init_subclass__(cls, **kwargs) -> None:
        """Called when a class inherits from BaseVectorStoreConfig."""
        super().__init_subclass__(**kwargs)
        cls._register_provider()
    
    @classmethod
    def __pydantic_init_subclass__(cls, **kwargs) -> None:
        """Called by Pydantic when a class inherits from BaseVectorStoreConfig."""
        super().__pydantic_init_subclass__(**kwargs)
        cls._register_provider()
    
    @classmethod
    def get_provider_config_cls(cls, provider: str) -> Optional[type["BaseVectorStoreConfig"]]:
        """Get the config class for a specific provider."""
        return cls._registry.get(provider)
    
    @classmethod
    def get_provider_class_path(cls, provider: str) -> Optional[str]:
        """Get the class path for a specific provider."""
        return cls._class_paths.get(provider)
    
    @classmethod
    def has_provider(cls, provider: str) -> bool:
        """Check if a provider is registered."""
        return provider in cls._registry
    
    def to_component_dict(self) -> Dict[str, Any]:
        """
        Convert config to component dictionary format.
        
        Returns:
            Dict with 'provider' and 'config' keys
        """
        return {
            "provider": self._provider_name,
            "config": self.model_dump(exclude_none=True)
        }


class BaseGraphStoreConfig(BaseVectorStoreConfig):
    """
    Base configuration class for all graph store providers.
    
    Inherits from BaseVectorStoreConfig to reuse connection and vector parameters.
    Adds graph-specific fields like max_hops.
    
    Environment variable priority (via AliasChoices):
    1. GRAPH_STORE_* (highest priority)
    2. OCEANBASE_* (fallback to VectorStore config)
    3. Default values
    """
    
    model_config = settings_config("GRAPH_STORE_", extra="allow", env_file=None)
    
    _provider_name: ClassVar[Optional[str]] = None
    _class_path: ClassVar[Optional[str]] = None
    _registry: ClassVar[dict[str, type["BaseGraphStoreConfig"]]] = {}
    _class_paths: ClassVar[dict[str, str]] = {}
    
    # Override connection fields with GRAPH_STORE_ fallback aliases
    host: str = Field(
        default="127.0.0.1",
        validation_alias=AliasChoices(
            "host",
            "GRAPH_STORE_HOST",    # Priority 1
            "OCEANBASE_HOST",      # Priority 2 (fallback)
        ),
        description="Database server host"
    )
    
    port: str = Field(
        default="2881",
        validation_alias=AliasChoices(
            "port",
            "GRAPH_STORE_PORT",
            "OCEANBASE_PORT",
        ),
        description="Database server port"
    )
    
    user: str = Field(
        default="root@test",
        validation_alias=AliasChoices(
            "GRAPH_STORE_USER",
            "OCEANBASE_USER",
            "user", # avoid using system USER environment variable first
        ),
        description="Database username"
    )
    
    password: str = Field(
        default="",
        validation_alias=AliasChoices(
            "password",
            "GRAPH_STORE_PASSWORD",
            "OCEANBASE_PASSWORD",
        ),
        description="Database password"
    )
    
    db_name: str = Field(
        default="test",
        validation_alias=AliasChoices(
            "db_name",
            "GRAPH_STORE_DB_NAME",
            "OCEANBASE_DATABASE",
        ),
        description="Database name"
    )
    
    # Override vector configuration fields
    vidx_metric_type: str = Field(
        default="l2",
        validation_alias=AliasChoices(
            "vidx_metric_type",
            "GRAPH_STORE_VECTOR_METRIC_TYPE",
            "OCEANBASE_VECTOR_METRIC_TYPE",
        ),
        description="Distance metric (l2, inner_product, cosine)"
    )
    
    index_type: str = Field(
        default="HNSW",
        validation_alias=AliasChoices(
            "index_type",
            "GRAPH_STORE_INDEX_TYPE",
            "OCEANBASE_INDEX_TYPE",
        ),
        description="Type of vector index (HNSW, IVF, FLAT, etc.)"
    )
    
    embedding_model_dims: Optional[int] = Field(
        default=None,
        validation_alias=AliasChoices(
            "embedding_model_dims",
            "GRAPH_STORE_EMBEDDING_MODEL_DIMS",
            "OCEANBASE_EMBEDDING_MODEL_DIMS",
        ),
        description="Dimension of embedding vectors"
    )
    
    # Graph-specific fields
    max_hops: int = Field(
        default=3,
        validation_alias=AliasChoices(
            "max_hops",
            "GRAPH_STORE_MAX_HOPS",
        ),
        description="Maximum number of hops for multi-hop graph search"
    )
    
    # GraphStore metadata fields (from GraphStoreConfig)
    # Note: BaseLLMConfig is imported lazily to avoid circular imports
    llm: Optional[Any] = Field(
        default=None,
        description="LLM configuration for querying the graph store (overrides global LLM)"
    )
    custom_prompt: Optional[str] = Field(
        default=None,
        description="Custom prompt to fetch entities from the given text"
    )
    custom_extract_relations_prompt: Optional[str] = Field(
        default=None,
        description="Custom prompt for extracting relations from text"
    )
    custom_update_graph_prompt: Optional[str] = Field(
        default=None,
        description="Custom prompt for updating graph memories"
    )
    custom_delete_relations_prompt: Optional[str] = Field(
        default=None,
        description="Custom prompt for deleting relations"
    )
    
    @classmethod
    def _register_provider(cls) -> None:
        """Register provider in the global registry."""
        provider = getattr(cls, "_provider_name", None)
        class_path = getattr(cls, "_class_path", None)
        if provider:
            BaseGraphStoreConfig._registry[provider] = cls
            if class_path:
                BaseGraphStoreConfig._class_paths[provider] = class_path
    
    def __init_subclass__(cls, **kwargs) -> None:
        """Called when a class inherits from BaseGraphStoreConfig."""
        super().__init_subclass__(**kwargs)
        cls._register_provider()
    
    @classmethod
    def __pydantic_init_subclass__(cls, **kwargs) -> None:
        """Called by Pydantic when a class inherits from BaseGraphStoreConfig."""
        super().__pydantic_init_subclass__(**kwargs)
        cls._register_provider()
    
    @classmethod
    def get_provider_config_cls(cls, provider: str) -> Optional[type["BaseGraphStoreConfig"]]:
        """Get the config class for a specific provider."""
        return cls._registry.get(provider)
    
    @classmethod
    def get_provider_class_path(cls, provider: str) -> Optional[str]:
        """Get the class path for a specific provider."""
        return cls._class_paths.get(provider)
    
    @classmethod
    def has_provider(cls, provider: str) -> bool:
        """Check if a provider is registered."""
        return provider in cls._registry
    
    def to_component_dict(self) -> Dict[str, Any]:
        """
        Convert config to component dictionary format.
        
        Returns:
            Dict with 'provider' and 'config' keys
        """
        return {
            "provider": self._provider_name,
            "config": self.model_dump(exclude_none=True)
        }


from typing import Optional

from pydantic import AliasChoices, Field
from powermem.settings import settings_config

from powermem.storage.config.base import BaseVectorStoreConfig


class SQLiteConfig(BaseVectorStoreConfig):
    """Configuration for SQLite vector store."""
    
    _provider_name = "sqlite"
    _class_path = "powermem.storage.sqlite.sqlite_vector_store.SQLiteVectorStore"
    
    model_config = settings_config("VECTOR_STORE_", extra="forbid", env_file=None)
    
    database_path: str = Field(
        default="./data/powermem_dev.db",
        validation_alias=AliasChoices(
            "database_path",
            "SQLITE_PATH",
        ),
        description="Path to SQLite database file"
    )
    
    collection_name: str = Field(
        default="memories",
        validation_alias=AliasChoices(
            "collection_name",
            "SQLITE_COLLECTION",
        ),
        description="Name of the collection/table"
    )
    
    enable_wal: bool = Field(
        default=True,
        validation_alias=AliasChoices(
            "enable_wal",
            "SQLITE_ENABLE_WAL",
        ),
        description="Enable Write-Ahead Logging for better concurrency"
    )
    
    timeout: int = Field(
        default=30,
        validation_alias=AliasChoices(
            "timeout",
            "SQLITE_TIMEOUT",
        ),
        description="Connection timeout in seconds"
    )




from typing import Any, Optional

from pydantic import AliasChoices, Field, model_validator
from powermem.settings import settings_config

from powermem.storage.config.base import BaseVectorStoreConfig


class PGVectorConfig(BaseVectorStoreConfig):
    _provider_name = "pgvector"
    _class_path = "powermem.storage.pgvector.pgvector.PGVectorStore"
    
    model_config = settings_config("VECTOR_STORE_", extra="forbid", env_file=None)

    dbname: str = Field(
        default="postgres",
        validation_alias=AliasChoices(
            "dbname",
            "POSTGRES_DATABASE",
        ),
        description="Default name for the database"
    )
    
    collection_name: str = Field(
        default="power_mem",
        validation_alias=AliasChoices(
            "collection_name",
            "POSTGRES_COLLECTION",
        ),
        description="Default name for the collection"
    )
    
    embedding_model_dims: Optional[int] = Field(
        default=1536,
        validation_alias=AliasChoices(
            "embedding_model_dims",
            "POSTGRES_EMBEDDING_MODEL_DIMS",
        ),
        description="Dimensions of the embedding model"
    )
    
    user: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "POSTGRES_USER",
            "user", # avoid using system USER environment variable first
        ),
        description="Database user"
    )
    
    password: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "password",
            "POSTGRES_PASSWORD",
        ),
        description="Database password"
    )
    
    host: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "host",
            "POSTGRES_HOST",
        ),
        description="Database host. Default is 127.0.0.1"
    )
    
    port: Optional[int] = Field(
        default=None,
        validation_alias=AliasChoices(
            "port",
            "POSTGRES_PORT",
        ),
        description="Database port. Default is 5432"
    )
    
    diskann: Optional[bool] = Field(
        default=False,
        validation_alias=AliasChoices(
            "diskann",
            "POSTGRES_DISKANN",
        ),
        description="Use diskann for approximate nearest neighbors search"
    )
    
    hnsw: Optional[bool] = Field(
        default=True,
        validation_alias=AliasChoices(
            "hnsw",
            "POSTGRES_HNSW",
        ),
        description="Use hnsw for faster search"
    )
    
    minconn: Optional[int] = Field(
        default=1,
        description="Minimum number of connections in the pool"
    )
    
    maxconn: Optional[int] = Field(
        default=5,
        description="Maximum number of connections in the pool"
    )
    
    sslmode: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "sslmode",
            "DATABASE_SSLMODE",
        ),
        description="SSL mode for PostgreSQL connection"
    )
    
    connection_string: Optional[str] = Field(
        default=None,
        description="PostgreSQL connection string"
    )
    
    connection_pool: Optional[Any] = Field(
        default=None,
        description="psycopg connection pool object"
    )

    @model_validator(mode="before")
    @classmethod
    def check_auth_and_connection(cls, values):
        if values.get("connection_pool") is not None:
            return values
        if values.get("connection_string") is not None:
            return values
        user, password = values.get("user"), values.get("password")
        host, port = values.get("host"), values.get("port")
        if user is not None or password is not None:
            if not user or not password:
                raise ValueError("Both 'user' and 'password' must be provided.")
        if host is not None or port is not None:
            if not host or not port:
                raise ValueError("Both 'host' and 'port' must be provided.")
        return values



"""
Setup utilities for powermem

This module provides setup functions compatible with initialization,
"""

import json
import os
import uuid
from pathlib import Path
from typing import Optional, Dict, Any
import logging

from pydantic import Field, field_validator
from pydantic_settings import BaseSettings

from ..settings import settings_config

logger = logging.getLogger(__name__)

# Set up the directory path
VECTOR_ID = str(uuid.uuid4())
class _LocalSettings(BaseSettings):
    model_config = settings_config()

    powermem_dir: Path = Field(
        default=Path.home() / ".powermem", validation_alias="POWERMEM_DIR"
    )

    @field_validator("powermem_dir", mode="before")
    @classmethod
    def expand_user_path(cls, value: object) -> object:
        if isinstance(value, str):
            return Path(os.path.expanduser(value))
        return value


def get_powermem_dir() -> Path:
    return _LocalSettings().powermem_dir


powermem_dir = get_powermem_dir()
os.makedirs(powermem_dir, exist_ok=True)


def setup_config():
    """Setup configuration file."""
    config_path = os.path.join(powermem_dir, "config.json")
    if not os.path.exists(config_path):
        user_id = str(uuid.uuid4())
        config = {"user_id": user_id}
        with open(config_path, "w") as config_file:
            json.dump(config, config_file, indent=4)


def get_user_id() -> str:
    """Get or create user ID."""
    config_path = os.path.join(powermem_dir, "config.json")
    if not os.path.exists(config_path):
        setup_config()
    
    try:
        with open(config_path, "r") as config_file:
            config = json.load(config_file)
            user_id = config.get("user_id")
            return user_id
    except Exception:
        return "anonymous_user"


def from_config(config: Optional[Dict[str, Any]] = None, **kwargs):
    """
    Create Memory instance from configuration.

    Deprecated: prefer `create_memory()` or `auto_config()`.
    
    powermem now uses field names natively: 'embedder' and 'vector_store'.
    
    Args:
        config: Configuration dictionary
               - llm: LLM provider configuration
               - embedder: Embedder configuration
               - vector_store: Vector store config (uses OceanBase)
        **kwargs: Additional parameters
    
    Returns:
        Memory instance
        
    Example:
        ```python
        from powermem import from_config
        
        memory = from_config({
            "llm": {"provider": "openai", "config": {"api_key": "..."}},
            "embedder": {"provider": "openai", "config": {"api_key": "..."}},
            "vector_store": {"provider": "oceanbase", "config": {...}},
        })
        ```
    """
    from ..core.memory import Memory
    
    if config is None:
        # Use auto config from environment
        from ..config_loader import auto_config
        config = auto_config()
    
    converted_config = _convert_legacy_to_mem_config(config)
    
    return Memory(config=converted_config, **kwargs)


def _convert_legacy_to_mem_config(config: Dict[str, Any]) -> Dict[str, Any]:
    """
    Convert legacy powermem config format.
    
    Now powermem uses field names natively, so we only convert legacy format.
    
    Args:
        config: Legacy powermem configuration dictionary
        
    Returns:
        configuration dictionary
    """
    if "embedder" in config or "vector_store" in config:
        return config
    
    converted = {}
    
    # LLM stays the same
    if "llm" in config:
        converted["llm"] = config["llm"]
    
    # Convert embedding to embedder
    if "embedding" in config:
        converted["embedder"] = config["embedding"]
    
    # Convert database to vector_store
    if "database" in config:
        db_config = config["database"]
        converted["vector_store"] = {
            "provider": db_config.get("provider", "oceanbase"),
            "config": db_config.get("config", {})
        }
    else:
        converted["vector_store"] = {
            "provider": "oceanbase",
            "config": {}
        }
    
    return converted


def get_or_create_user_id(vector_store=None) -> str:
    """
    Store user_id in vector store and return it.
    
    Args:
        vector_store: Optional vector store instance
        
    Returns:
        User ID
    """
    user_id = get_user_id()
    
    if vector_store is None:
        return user_id
    
    # Try to get existing user_id from vector store
    try:
        existing = vector_store.get(vector_id=user_id)
        if existing and hasattr(existing, "payload") and existing.payload and "user_id" in existing.payload:
            return existing.payload["user_id"]
    except Exception:
        pass
    
    # If we get here, we need to insert the user_id
    try:
        dims = getattr(vector_store, "embedding_model_dims", 1536)
        vector_store.insert(
            vectors=[[0.1] * dims], 
            payloads=[{"user_id": user_id, "type": "user_identity"}], 
            ids=[user_id]
        )
    except Exception:
        pass
    
    return user_id



"""
Storage configuration module

This module provides configuration classes for different storage providers.
"""

from .base import BaseVectorStoreConfig, BaseGraphStoreConfig
from .oceanbase import OceanBaseConfig, OceanBaseGraphConfig
from .pgvector import PGVectorConfig
from .sqlite import SQLiteConfig

__all__ = [
    "BaseVectorStoreConfig",
    "BaseGraphStoreConfig",
    "OceanBaseConfig",
    "OceanBaseGraphConfig",
    "PGVectorConfig",
    "SQLiteConfig",
]



from typing import Any, Dict, Optional, Union

import httpx
from pydantic import AliasChoices, Field

from powermem.integrations.embeddings.config.base import BaseEmbedderConfig
from powermem.settings import settings_config


class OpenAIEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "openai"
    _class_path = "powermem.integrations.embeddings.openai.OpenAIEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    openai_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "OPENAI_EMBEDDING_BASE_URL",
            "OPEN_EMBEDDING_BASE_URL",
        ),
    )


class QwenEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "qwen"
    _class_path = "powermem.integrations.embeddings.qwen.QwenEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "QWEN_API_KEY",
            "DASHSCOPE_API_KEY",
            "EMBEDDING_API_KEY",
        ),
    )
    model: Optional[str] = Field(default=None)
    dashscope_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "dashscope_base_url",
            "QWEN_EMBEDDING_BASE_URL",
        ),
    )
    memory_add_embedding_type: Optional[str] = Field(default=None)
    memory_update_embedding_type: Optional[str] = Field(default=None)
    memory_search_embedding_type: Optional[str] = Field(default=None)


class SiliconFlowEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "siliconflow"
    _class_path = "powermem.integrations.embeddings.siliconflow.SiliconFlowEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    siliconflow_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices("SILICONFLOW_EMBEDDING_BASE_URL"),
    )


class HuggingFaceEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "huggingface"
    _class_path = "powermem.integrations.embeddings.huggingface.HuggingFaceEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    huggingface_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices("HUGGINFACE_EMBEDDING_BASE_URL"),
    )
    model_kwargs: Dict[str, Any] = Field(default_factory=dict)


class OllamaEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "ollama"
    _class_path = "powermem.integrations.embeddings.ollama.OllamaEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    ollama_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices("OLLAMA_EMBEDDING_BASE_URL"),
    )


class LMStudioEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "lmstudio"
    _class_path = "powermem.integrations.embeddings.lmstudio.LMStudioEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    lmstudio_base_url: Optional[str] = Field(
        default="http://localhost:1234/v1",
        validation_alias=AliasChoices("LMSTUDIO_EMBEDDING_BASE_URL"),
    )


class AzureOpenAIEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "azure_openai"
    _class_path = "powermem.integrations.embeddings.azure_openai.AzureOpenAIEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "AZURE_OPENAI_API_KEY",
            "AZURE_API_KEY",
            "EMBEDDING_API_KEY",
        ),
    )
    azure_deployment: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices("AZURE_DEPLOYMENT"),
    )
    azure_endpoint: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices("AZURE_ENDPOINT", "AZURE_OPENAI_ENDPOINT"),
    )
    api_version: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices("AZURE_API_VERSION"),
    )
    default_headers: Optional[Dict[str, str]] = Field(default=None)
    http_client_proxies: Optional[Union[Dict[str, Any], str]] = Field(default=None)
    http_client: Optional[httpx.Client] = Field(default=None, exclude=True)

    def model_post_init(self, __context: Any) -> None:
        if self.http_client_proxies:
            self.http_client = httpx.Client(proxies=self.http_client_proxies)


class GeminiEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "gemini"
    _class_path = "powermem.integrations.embeddings.gemini.GoogleGenAIEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    output_dimensionality: Optional[int] = Field(default=None)


class VertexAIEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "vertexai"
    _class_path = "powermem.integrations.embeddings.vertexai.VertexAIEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    vertex_credentials_json: Optional[str] = Field(default=None)
    memory_add_embedding_type: Optional[str] = Field(default=None)
    memory_update_embedding_type: Optional[str] = Field(default=None)
    memory_search_embedding_type: Optional[str] = Field(default=None)


class TogetherEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "together"
    _class_path = "powermem.integrations.embeddings.together.TogetherEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)


class AWSBedrockEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "aws_bedrock"
    _class_path = "powermem.integrations.embeddings.aws_bedrock.AWSBedrockEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    aws_access_key_id: Optional[str] = Field(default=None)
    aws_secret_access_key: Optional[str] = Field(default=None)
    aws_region: Optional[str] = Field(default=None)


class ZaiEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "zai"
    _class_path = "powermem.integrations.embeddings.zai.ZaiEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[str] = Field(default=None)
    zai_base_url: Optional[str] = Field(default=None)


class LangchainEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "langchain"
    _class_path = "powermem.integrations.embeddings.langchain.LangchainEmbedding"

    model_config = settings_config("EMBEDDING_", extra="forbid", env_file=None)

    model: Optional[Any] = Field(default=None)


class MockEmbeddingConfig(BaseEmbedderConfig):
    _provider_name = "mock"
    _class_path = "powermem.integrations.embeddings.mock.MockEmbeddings"

    model_config = settings_config("EMBEDDING_", extra="allow", env_file=None)


class CustomEmbeddingConfig(BaseEmbedderConfig):
    model_config = settings_config("EMBEDDING_", extra="allow", env_file=None)



import os
from typing import Literal, Optional

from powermem.integrations.embeddings.config.sparse_base import BaseSparseEmbedderConfig
from powermem.integrations.embeddings.sparse_base import SparseEmbeddingBase

try:
    from dashscope import TextEmbedding
    from dashscope.api_entities.dashscope_response import DashScopeAPIResponse
except ImportError:
    TextEmbedding = None
    DashScopeAPIResponse = None
import dashscope


class QwenSparseEmbedding(SparseEmbeddingBase):
    def __init__(self, config: Optional[BaseSparseEmbedderConfig] = None):
        super().__init__(config)

        # Set default model and dimensions
        self.config.model = self.config.model or "text-embedding-v4"
        self.config.embedding_dims = self.config.embedding_dims or 1536

        # Check if dashscope is available
        if TextEmbedding is None:
            raise ImportError(
                "DashScope SDK is not installed. Please install it with: pip install dashscope"
            )

        # Set API key
        api_key = self.config.api_key or os.getenv("DASHSCOPE_API_KEY")
        if not api_key:
            raise ValueError(
                "API key is required. Set DASHSCOPE_API_KEY environment variable or pass api_key in config.")

        # Set API key for DashScope SDK
        dashscope.api_key = api_key

        # Set base URL (if needed)
        base_url = (
            self.config.base_url
            or os.getenv("DASHSCOPE_BASE_URL")
            or "https://dashscope.aliyuncs.com/api/v1"
        )
        if base_url:
            os.environ["DASHSCOPE_BASE_URL"] = base_url

    def embed_sparse(self, text: str, memory_action: Optional[Literal["add", "search", "update"]] = None) -> dict[int, float]:
        """
        Get the embedding for the given text using Qwen.

        Args:
            text (str): The text to embed.
            memory_action (optional): The type of embedding to use. Must be one of "add", "search", or "update". Defaults to None.
        Returns:
            list: The embedding vector.
        """
        # Clean text
        text = text.replace("\n", " ").strip()

        # Determine embedding type based on memory action
        if memory_action == "add":
            embedding_type = "document"
        elif memory_action == "search":
            embedding_type =  "query"
        elif memory_action == "update":
            embedding_type = "document"
        else:
            # Default to "document" if memory_action is None or unknown
            embedding_type = "document"

        try:
            params = {
                "model": self.config.model,
                "input": text,
                "output_type": "sparse",
            }

            # Add dimension parameter if specified
            if hasattr(self.config, 'embedding_dims') and self.config.embedding_dims:
                params["dimension"] = self.config.embedding_dims

            # Add embedding type (always set, either from config or default)
            params["text_type"] = embedding_type

            # Call the API
            response = TextEmbedding.call(**params)

            if response.status_code != 200:
                raise Exception(f"API request failed with status {response.status_code}: {response.message}")

            # Extract sparse embedding vector
            # response.output is a dict, not an object with attributes
            if isinstance(response.output, dict) and 'embeddings' in response.output:
                sparse_embedding_list = response.output['embeddings'][0].get('sparse_embedding', [])
                # Convert sparse_embedding list to dict format: {index: value}
                embedding = {item['index']: item['value'] for item in sparse_embedding_list}
            else:
                # Fallback for different response structures
                sparse_embedding_list = response.output.get('embeddings', [{}])[0].get('sparse_embedding', [])
                embedding = {item.get('index', 0): item.get('value', 0.0) for item in sparse_embedding_list if 'index' in item and 'value' in item}

            return embedding

        except Exception as e:
            raise Exception(f"Failed to generate embedding: {e}")



"""
Configuration management for PowerMem API Server.
"""

from __future__ import annotations
from typing import List, Optional

from pydantic import Field, field_validator
from pydantic_settings import BaseSettings

from powermem.settings import settings_config


def _parse_boolish(value: object) -> object:
    """
    Backward-compatible boolean parsing.

    Historically we accepted values like: true/1/yes/on/enabled.
    `pydantic` already accepts many truthy strings, but "enabled"/"disabled" are not
    guaranteed across versions, so we normalize explicitly.
    """

    if value is None or isinstance(value, bool):
        return value

    if isinstance(value, str):
        text = value.strip().lower()
        if text in {"1", "true", "t", "yes", "y", "on", "enabled"}:
            return True
        if text in {"0", "false", "f", "no", "n", "off", "disabled"}:
            return False

    return value


class ServerSettings(BaseSettings):
    model_config = settings_config(
        env_prefix="POWERMEM_SERVER_",
        extra="ignore",
    )

    # Server settings
    host: str = Field(default="0.0.0.0")
    port: int = Field(default=8000)
    workers: int = Field(default=4)
    reload: bool = Field(default=False)

    # Authentication settings
    auth_enabled: bool = Field(default=True)
    api_keys: str = Field(default="")

    # Rate limiting settings
    rate_limit_enabled: bool = Field(default=True)
    rate_limit_per_minute: int = Field(default=100)

    # Logging settings
    log_level: str = Field(default="INFO")
    log_format: str = Field(default="json")
    log_file: Optional[str] = Field(default="server.log")

    # API settings
    api_title: str = Field(default="PowerMem API")
    api_version: str = Field(default="v1")
    api_description: str = Field(
        default="PowerMem HTTP API Server - Intelligent Memory System"
    )

    # CORS settings
    cors_enabled: bool = Field(default=True)
    cors_origins: str = Field(default="*")

    @field_validator(
        "reload",
        "auth_enabled",
        "rate_limit_enabled",
        "cors_enabled",
        mode="before",
    )
    @classmethod
    def normalize_bool_fields(cls, value: object) -> object:
        return _parse_boolish(value)

    @field_validator("log_file", mode="before")
    @classmethod
    def normalize_log_file(cls, value: object) -> Optional[str]:
        if value is None:
            return None
        if isinstance(value, str):
            stripped = value.strip()
            return stripped or None
        return value  # type: ignore[return-value]

    @field_validator("log_level", mode="before")
    @classmethod
    def normalize_log_level(cls, value: object) -> object:
        if isinstance(value, str):
            stripped = value.strip()
            return stripped.upper() if stripped else value
        return value

    @field_validator("log_format", mode="before")
    @classmethod
    def normalize_log_format(cls, value: object) -> object:
        if isinstance(value, str):
            stripped = value.strip()
            return stripped.lower() if stripped else value
        return value

    def get_api_keys_list(self) -> List[str]:
        """Get list of API keys"""
        if not self.api_keys:
            return []
        return [key.strip() for key in self.api_keys.split(",") if key.strip()]

    def get_cors_origins_list(self) -> List[str]:
        """Get list of CORS origins"""
        if self.cors_origins == "*":
            return ["*"]
        return [
            origin.strip() for origin in self.cors_origins.split(",") if origin.strip()
        ]


config = ServerSettings()



from typing import Optional

from pydantic import AliasChoices, Field

from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.settings import settings_config


class VllmConfig(BaseLLMConfig):
    """
    Configuration class for vLLM-specific parameters.
    Inherits from BaseLLMConfig and adds vLLM-specific settings.
    """

    _provider_name = "vllm"
    _class_path = "powermem.integrations.llm.vllm.VllmLLM"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)

    # vLLM-specific fields
    vllm_base_url: Optional[str] = Field(
        default="http://localhost:8000/v1",
        validation_alias=AliasChoices(
            "vllm_base_url",
            "VLLM_LLM_BASE_URL",
        ),
        description="vLLM base URL"
    )



"""
Sparse Vector Upgrade Script

This script is used to add sparse vector support to existing OceanBase tables:
- Add sparse_embedding column (SPARSE_VECTOR type)
- Create sparse_embedding_idx index
"""
import logging
from typing import Dict, Any, Tuple

from pyobvector import SPARSE_VECTOR, IndexParam, ObVecClient
from sqlalchemy import text, Column

logger = logging.getLogger(__name__)


def _validate_and_parse_config(config: Dict[str, Any]) -> Tuple[ObVecClient, str]:
    """
    Validate configuration and create database connection
    
    Args:
        config: PowerMem configuration dictionary
        
    Returns:
        Tuple[ObVecClient, str]: (database connection object, table name)
        
    Raises:
        TypeError: Invalid configuration type
        ValueError: Invalid configuration content
        RuntimeError: Connection failed
    """
    from powermem.configs import MemoryConfig
    
    # 1. Validate config type
    if not isinstance(config, (dict, MemoryConfig)):
        raise TypeError(
            f"Expected dict or MemoryConfig instance, got {type(config).__name__}. "
            f"Please pass a config dict:\n"
            f"  from powermem import auto_config\n"
            f"  config = auto_config()\n"
            f"  ScriptManager.run('upgrade-sparse-vector', config)"
        )
    
    # 2. Convert to dict if needed
    config_dict = config.model_dump() if isinstance(config, MemoryConfig) else config
    
    # 3. Validate vector_store config
    vector_store_config = config_dict.get('vector_store')
    if not vector_store_config:
        raise ValueError(
            "No vector_store configuration found in config. "
            "Please ensure config contains 'vector_store' section."
        )
    
    # 4. Validate storage type
    storage_type = vector_store_config.get('provider', '').lower()
    if storage_type != 'oceanbase':
        raise ValueError(
            f"Vector store type is not OceanBase (got '{storage_type}'). "
            f"This upgrade script only supports OceanBase storage. "
            f"Please configure vector_store.provider as 'oceanbase'."
        )
    
    # 5. Extract connection parameters
    ob_config = vector_store_config.get('config', {})
    connection_args = ob_config.get('connection_args', {})
    host = connection_args.get('host')
    port = connection_args.get('port')
    user = connection_args.get('user')
    password = connection_args.get('password')
    db_name = connection_args.get('db_name')
    collection_name = ob_config.get('collection_name', 'power_mem')
    
    # 6. Validate required parameters
    if not all([host, port, user, db_name]):
        missing = []
        if not host: missing.append('host')
        if not port: missing.append('port')
        if not user: missing.append('user')
        if not db_name: missing.append('db_name')
        raise ValueError(
            f"Missing required OceanBase connection parameters: {', '.join(missing)}. "
            f"Please ensure config contains 'vector_store.config.connection_args.{{host, port, user, db_name}}'."
        )
    
    # 7. Create database connection
    try:
        logger.info(f"Connecting to OceanBase at {host}:{port}...")
        obvector = ObVecClient(
            uri=f"{host}:{port}",
            user=user,
            password=password or "",
            db_name=db_name
        )
        logger.info(f"Connected successfully to database '{db_name}'")
        return obvector, collection_name
    except Exception as e:
        raise RuntimeError(f"Failed to connect to OceanBase: {e}") from e


def upgrade_sparse_vector(config: Dict[str, Any]) -> bool:
    """
    Add sparse vector support to OceanBase table
    
    This function checks and adds the columns and indexes required for sparse vectors.
    It is idempotent and can be safely called multiple times
    (existing columns and indexes will be skipped).
    
    Args:
        config: PowerMem configuration dictionary containing OceanBase connection settings
        
    Returns:
        bool: Returns True on success, False on failure
        
    Raises:
        TypeError: Invalid configuration type
        ValueError: Storage type is not OceanBase or config is invalid
        RuntimeError: Database version does not support sparse vectors
    """
    from powermem.utils.oceanbase_util import OceanBaseUtil
    
    # Validate config and create connection
    obvector, collection_name = _validate_and_parse_config(config)
    
    logger.info(f"Starting sparse vector upgrade for table '{collection_name}'")
    
    # Check if database version supports sparse vectors
    if not OceanBaseUtil.check_sparse_vector_version_support(obvector):
        raise RuntimeError(
            "Database version does not support SPARSE_VECTOR type. "
            "Sparse vector requires seekdb or OceanBase >= 4.5.0. "
            "Please upgrade your database before running this script."
        )
    
    # Check if table exists
    if not OceanBaseUtil.check_table_exists(obvector, collection_name):
        raise RuntimeError(
            f"Table '{collection_name}' does not exist. "
            f"Please create the table first by initializing Memory with include_sparse=False, "
            f"then run this upgrade script."
        )
    
    try:
        # Add sparse_embedding column (if not exists)
        _add_sparse_embedding_column(obvector, collection_name)
        
        # Create sparse_embedding index (if not exists)
        _create_sparse_embedding_index(obvector, collection_name)
        
        logger.info(f"Sparse vector upgrade completed successfully for table '{collection_name}'")
        return True
        
    except RuntimeError:
        raise
    except Exception as e:
        logger.error(f"Sparse vector upgrade failed: {e}", exc_info=True)
        raise RuntimeError(f"Sparse vector upgrade failed: {e}") from e


def _add_sparse_embedding_column(obvector: ObVecClient, table_name: str) -> None:
    """Add sparse_embedding column to table"""
    from powermem.utils.oceanbase_util import OceanBaseUtil
    
    if OceanBaseUtil.check_column_exists(obvector, table_name, 'sparse_embedding'):
        logger.info("sparse_embedding column already exists, skipping")
        return
    
    logger.info(f"Adding sparse_embedding column to table '{table_name}'")
    try:
        obvector.add_columns(
            table_name=table_name,
            columns=[Column('sparse_embedding', SPARSE_VECTOR)]
        )
        logger.info("sparse_embedding column added successfully")
    except Exception as e:
        error_str = str(e).lower()
        if '1060' in str(e) or 'already exists' in error_str or 'duplicate column' in error_str:
            logger.info("sparse_embedding column already exists (race condition), skipping")
        else:
            raise RuntimeError(f"Failed to add sparse_embedding column: {e}") from e


def _create_sparse_embedding_index(obvector: ObVecClient, table_name: str) -> None:
    """Create sparse vector index on table"""
    from powermem.utils.oceanbase_util import OceanBaseUtil
    
    if OceanBaseUtil.check_index_exists(obvector, table_name, 'sparse_embedding_idx'):
        logger.info("sparse_embedding_idx already exists, skipping")
        return
    
    logger.info(f"Creating sparse vector index on table '{table_name}'")
    try:
        vidx_param = IndexParam(
            field_name="sparse_embedding",
            index_type="daat",
            index_name="sparse_embedding_idx",
            metric_type="inner_product",
            sparse_index_type="sindi",
        )
        
        obvector.create_vidx_with_vec_index_param(
            table_name=table_name,
            vidx_param=vidx_param,
        )
        logger.info("sparse_embedding_idx created successfully")
    except Exception as e:
        error_str = str(e).lower()
        if '1061' in str(e) or 'already exists' in error_str or 'duplicate key' in error_str:
            logger.info("sparse_embedding_idx already exists (race condition), skipping")
        else:
            raise RuntimeError(f"Failed to create sparse_embedding_idx: {e}") from e


def downgrade_sparse_vector(config: Dict[str, Any]) -> bool:
    """
    Remove sparse vector support from OceanBase table (rollback operation)
    
    This function removes sparse vector columns and indexes.
    
    Warning: This operation will delete all sparse vector data and is irreversible!
    
    Args:
        config: PowerMem configuration dictionary containing OceanBase connection settings
        
    Returns:
        bool: Returns True on success, False on failure
        
    Raises:
        TypeError: Invalid configuration type
        ValueError: Storage type is not OceanBase or config is invalid
    """
    from powermem.utils.oceanbase_util import OceanBaseUtil
    
    # Validate config and create connection
    obvector, collection_name = _validate_and_parse_config(config)
    
    logger.info(f"Starting sparse vector downgrade for table '{collection_name}'")
    
    # Check if table exists
    if not OceanBaseUtil.check_table_exists(obvector, collection_name):
        logger.info(f"Table '{collection_name}' does not exist, nothing to downgrade")
        return True
    
    try:
        # Drop sparse_embedding index (if exists)
        _drop_sparse_embedding_index(obvector, collection_name)
        
        # Drop sparse_embedding column (if exists)
        _drop_sparse_embedding_column(obvector, collection_name)
        
        logger.info(f"Sparse vector downgrade completed successfully for table '{collection_name}'")
        return True
        
    except RuntimeError:
        raise
    except Exception as e:
        logger.error(f"Sparse vector downgrade failed: {e}", exc_info=True)
        raise RuntimeError(f"Sparse vector downgrade failed: {e}") from e


def _drop_sparse_embedding_index(obvector: ObVecClient, table_name: str) -> None:
    """Drop sparse vector index from table"""
    from powermem.utils.oceanbase_util import OceanBaseUtil
    
    if not OceanBaseUtil.check_index_exists(obvector, table_name, 'sparse_embedding_idx'):
        logger.info("sparse_embedding_idx does not exist, skipping")
        return
    
    logger.info(f"Dropping sparse vector index from table '{table_name}'")
    try:
        with obvector.engine.connect() as conn:
            conn.execute(text(f"DROP INDEX sparse_embedding_idx ON {table_name}"))
            conn.commit()
        logger.info("sparse_embedding_idx dropped successfully")
    except Exception as e:
        error_str = str(e).lower()
        if '1091' in str(e) or "doesn't exist" in error_str:
            logger.info("sparse_embedding_idx does not exist (race condition), skipping")
        else:
            raise RuntimeError(f"Failed to drop sparse_embedding_idx: {e}") from e


def _drop_sparse_embedding_column(obvector: ObVecClient, table_name: str) -> None:
    """Drop sparse_embedding column from table"""
    from powermem.utils.oceanbase_util import OceanBaseUtil
    
    if not OceanBaseUtil.check_column_exists(obvector, table_name, 'sparse_embedding'):
        logger.info("sparse_embedding column does not exist, skipping")
        return
    
    logger.info(f"Dropping sparse_embedding column from table '{table_name}'")
    try:
        with obvector.engine.connect() as conn:
            conn.execute(text(f"ALTER TABLE {table_name} DROP COLUMN sparse_embedding"))
            conn.commit()
        logger.info("sparse_embedding column dropped successfully")
    except Exception as e:
        error_str = str(e).lower()
        if '1091' in str(e) or "doesn't exist" in error_str:
            logger.info("sparse_embedding column does not exist (race condition), skipping")
        else:
            raise RuntimeError(f"Failed to drop sparse_embedding column: {e}") from e



from typing import Any, ClassVar, Dict, Optional, Union

import httpx
from pydantic import Field
from pydantic_settings import BaseSettings

from powermem.settings import settings_config


class BaseLLMConfig(BaseSettings):
    """
    Base configuration for LLMs with only common parameters.
    Provider-specific configurations should be handled by separate config classes.

    This class contains only the parameters that are common across all LLM providers.
    For provider-specific parameters, use the appropriate provider config class.
    
    Now uses pydantic-settings for automatic environment variable loading.
    """

    model_config = settings_config("LLM_", extra="allow", env_file=None)

    # Registry for provider configurations
    _provider_name: ClassVar[Optional[str]] = None
    _class_path: ClassVar[Optional[str]] = None
    _registry: ClassVar[dict[str, type["BaseLLMConfig"]]] = {}
    _class_paths: ClassVar[dict[str, str]] = {}

    # Field definitions
    model: Optional[Union[str, Dict]] = Field(
        default=None,
        description="The model identifier to use (e.g., 'gpt-4o-mini', 'claude-3-5-sonnet-20240620'). "
                    "Defaults to None (will be set by provider-specific configs)"
    )
    
    temperature: float = Field(
        default=0.1,
        description="Controls the randomness of the model's output. "
                    "Higher values (closer to 1) make output more random, lower values make it more deterministic. "
                    "Range: 0.0 to 2.0"
    )
    
    api_key: Optional[str] = Field(
        default=None,
        description="API key for the LLM provider. If None, will try to get from environment variables"
    )
    
    max_tokens: int = Field(
        default=2000,
        description="Maximum number of tokens to generate in the response. "
                    "Range: 1 to 4096 (varies by model)"
    )
    
    top_p: float = Field(
        default=0.1,
        description="Nucleus sampling parameter. Controls diversity via nucleus sampling. "
                    "Higher values (closer to 1) make word selection more diverse. "
                    "Range: 0.0 to 1.0"
    )
    
    top_k: int = Field(
        default=1,
        description="Top-k sampling parameter. Limits the number of tokens considered for each step. "
                    "Higher values make word selection more diverse. "
                    "Range: 1 to 40"
    )
    
    enable_vision: bool = Field(
        default=False,
        description="Whether to enable vision capabilities for the model. "
                    "Only applicable to vision-enabled models"
    )
    
    vision_details: Optional[str] = Field(
        default="auto",
        description="Level of detail for vision processing. Options: 'low', 'high', 'auto'"
    )
    
    http_client_proxies: Optional[Union[Dict, str]] = Field(
        default=None,
        description="Proxy settings for HTTP client. Can be a dict or string"
    )
    
    http_client: Optional[httpx.Client] = Field(
        default=None,
        exclude=True,
        description="HTTP client instance (automatically initialized from http_client_proxies)"
    )

    @classmethod
    def _register_provider(cls) -> None:
        """Register provider in the global registry."""
        provider = getattr(cls, "_provider_name", None)
        class_path = getattr(cls, "_class_path", None)
        if provider:
            BaseLLMConfig._registry[provider] = cls
            if class_path:
                BaseLLMConfig._class_paths[provider] = class_path

    def __init_subclass__(cls, **kwargs) -> None:
        """Called when a class inherits from BaseLLMConfig."""
        super().__init_subclass__(**kwargs)
        cls._register_provider()

    @classmethod
    def __pydantic_init_subclass__(cls, **kwargs) -> None:
        """Called by Pydantic when a class inherits from BaseLLMConfig."""
        super().__pydantic_init_subclass__(**kwargs)
        cls._register_provider()

    @classmethod
    def get_provider_config_cls(cls, provider: str) -> Optional[type["BaseLLMConfig"]]:
        """Get the config class for a specific provider."""
        return cls._registry.get(provider)

    @classmethod
    def get_provider_class_path(cls, provider: str) -> Optional[str]:
        """Get the class path for a specific provider."""
        return cls._class_paths.get(provider)

    @classmethod
    def has_provider(cls, provider: str) -> bool:
        """Check if a provider is registered."""
        return provider in cls._registry

    def model_post_init(self, __context: Any) -> None:
        """Initialize http_client after model creation."""
        if self.http_client_proxies and not self.http_client:
            self.http_client = httpx.Client(proxies=self.http_client_proxies)

    def to_component_dict(self) -> Dict[str, Any]:
        """
        Convert config to component dictionary format.
        
        This method is used by MemoryConfig.to_dict() to serialize
        LLM configuration in a consistent format.
        
        Returns:
            Dict with 'provider' and 'config' keys
        """
        return {
            "provider": self._provider_name,
            "config": self.model_dump(exclude_none=True)
        }



"""
Provider-specific rerank configurations
"""
from typing import Optional

from pydantic import AliasChoices, Field

from powermem.integrations.rerank.config.base import BaseRerankConfig
from powermem.settings import settings_config


class QwenRerankConfig(BaseRerankConfig):
    """Configuration for Qwen (DashScope) rerank service"""
    
    _provider_name = "qwen"
    _class_path = "powermem.integrations.rerank.qwen.QwenRerank"
    
    model_config = settings_config("RERANK_", extra="forbid", env_file=None)
    
    # Override base fields with Qwen-specific aliases
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",  # Must include field name itself!
            "RERANK_API_KEY",
            "QWEN_API_KEY",
            "DASHSCOPE_API_KEY",
        ),
        description="API key for Qwen rerank service"
    )
    
    model: Optional[str] = Field(
        default="qwen3-rerank",
        description="Qwen rerank model name"
    )
    
    api_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_base_url",
            "RERANK_API_BASE_URL",
            "QWEN_RERANK_BASE_URL",
            "DASHSCOPE_BASE_URL",
        ),
        description="Base URL for Qwen/DashScope API"
    )


class JinaRerankConfig(BaseRerankConfig):
    """Configuration for Jina AI rerank service"""
    
    _provider_name = "jina"
    _class_path = "powermem.integrations.rerank.jina.JinaRerank"
    
    model_config = settings_config("RERANK_", extra="forbid", env_file=None)
    
    # Override base fields with Jina-specific aliases
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "RERANK_API_KEY",
            "JINA_API_KEY",
        ),
        description="API key for Jina AI"
    )
    
    model: Optional[str] = Field(
        default="jina-reranker-v3",
        description="Jina rerank model name"
    )
    
    api_base_url: Optional[str] = Field(
        default="https://api.jina.ai/v1/rerank",
        validation_alias=AliasChoices(
            "api_base_url",
            "RERANK_API_BASE_URL",
            "JINA_API_BASE_URL",
        ),
        description="Base URL for Jina AI rerank API"
    )


class ZaiRerankConfig(BaseRerankConfig):
    """Configuration for Zhipu AI rerank service"""
    
    _provider_name = "zai"
    _class_path = "powermem.integrations.rerank.zai.ZaiRerank"
    
    model_config = settings_config("RERANK_", extra="forbid", env_file=None)
    
    # Override base fields with Zhipu AI-specific aliases
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "RERANK_API_KEY",
            "ZAI_API_KEY",
        ),
        description="API key for Zhipu AI"
    )
    
    model: Optional[str] = Field(
        default="rerank",
        description="Zhipu AI rerank model name"
    )
    
    api_base_url: Optional[str] = Field(
        default="https://open.bigmodel.cn/api/paas/v4/rerank",
        validation_alias=AliasChoices(
            "api_base_url",
            "RERANK_API_BASE_URL",
            "ZAI_API_BASE_URL",
        ),
        description="Base URL for Zhipu AI rerank API"
    )


class GenericRerankConfig(BaseRerankConfig):
    """Configuration for generic rerank service"""
    
    _provider_name = "generic"
    _class_path = "powermem.integrations.rerank.generic.GenericRerank"
    
    model_config = settings_config("RERANK_", extra="forbid", env_file=None)
    
    # Generic uses base class default configuration



from typing import Any, Callable, List, Optional

from pydantic import AliasChoices, Field

from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.settings import settings_config


class OpenAIConfig(BaseLLMConfig):
    """
    Configuration class for OpenAI and OpenRouter-specific parameters.
    Inherits from BaseLLMConfig and adds OpenAI-specific settings.
    """

    _provider_name = "openai"
    _class_path = "powermem.integrations.llm.openai.OpenAILLM"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)

    # Override base fields with OpenAI-specific validation_alias
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "LLM_API_KEY",
            "OPENAI_API_KEY",
        ),
        description="OpenAI API key"
    )

    # OpenAI-specific fields
    openai_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "openai_base_url",
            "OPENAI_LLM_BASE_URL",
        ),
        description="OpenAI API base URL"
    )
    
    models: Optional[List[str]] = Field(
        default=None,
        description="List of models for OpenRouter"
    )
    
    route: Optional[str] = Field(
        default="fallback",
        description="OpenRouter route strategy"
    )
    
    openrouter_base_url: Optional[str] = Field(
        default=None,
        description="OpenRouter base URL"
    )
    
    site_url: Optional[str] = Field(
        default=None,
        description="Site URL for OpenRouter"
    )
    
    app_name: Optional[str] = Field(
        default=None,
        description="Application name for OpenRouter"
    )
    
    store: bool = Field(
        default=False,
        description="Whether to store conversations"
    )
    
    response_callback: Optional[Callable[[Any, dict, dict], None]] = Field(
        default=None,
        exclude=True,
        description="Optional callback for monitoring LLM responses"
    )



import json
import logging
import os
from typing import Dict, List, Optional, Union

from zai import ZhipuAiClient

from powermem.integrations.llm import LLMBase
from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.integrations.llm.config.zai import ZaiConfig
from powermem.utils.utils import extract_json


class ZaiLLM(LLMBase):
    """
    Zhipu AI (Z.ai) LLM implementation.
    
    Reference: https://docs.bigmodel.cn/cn/guide/develop/python/introduction
    
    Supported models:
    - glm-4.7: Latest language model
    - glm-4.6v: Vision understanding model
    - glm-4: General language model
    """

    def __init__(self, config: Optional[Union[BaseLLMConfig, ZaiConfig, Dict]] = None):
        # Convert to ZaiConfig if needed
        if config is None:
            config = ZaiConfig()
        elif isinstance(config, dict):
            config = ZaiConfig(**config)
        elif isinstance(config, BaseLLMConfig) and not isinstance(config, ZaiConfig):
            # Convert BaseLLMConfig to ZaiConfig
            config = ZaiConfig(
                model=config.model,
                temperature=config.temperature,
                api_key=config.api_key,
                max_tokens=config.max_tokens,
                top_p=config.top_p,
                top_k=config.top_k,
                enable_vision=config.enable_vision,
                vision_details=config.vision_details,
                http_client_proxies=config.http_client,
            )

        super().__init__(config)

        # Set default model if not specified
        if not self.config.model:
            self.config.model = "glm-4.7"

        # Get API key from config or environment
        api_key = self.config.api_key or os.getenv("ZAI_API_KEY")

        # Initialize Zhipu AI client
        self.client = ZhipuAiClient(api_key=api_key)

    def _parse_response(self, response, tools):
        """
        Process the response based on whether tools are used or not.

        Args:
            response: The raw response from API.
            tools: The list of tools provided in the request.

        Returns:
            str or dict: The processed response.
        """
        if tools:
            processed_response = {
                "content": response.choices[0].message.content,
                "tool_calls": [],
            }

            if response.choices[0].message.tool_calls:
                for tool_call in response.choices[0].message.tool_calls:
                    # Extract and validate arguments
                    arguments_str = extract_json(tool_call.function.arguments)

                    # Check if arguments are empty or whitespace only
                    if not arguments_str or arguments_str.strip() == "":
                        logging.warning(
                            f"Tool call '{tool_call.function.name}' has empty arguments. Skipping this tool call."
                        )
                        continue

                    # Try to parse JSON with error handling
                    try:
                        arguments = json.loads(arguments_str)
                    except json.JSONDecodeError as e:
                        logging.error(
                            f"Failed to parse tool call arguments for '{tool_call.function.name}': "
                            f"{arguments_str[:100]}... Error: {e}"
                        )
                        continue

                    processed_response["tool_calls"].append(
                        {
                            "name": tool_call.function.name,
                            "arguments": arguments,
                        }
                    )

            return processed_response
        else:
            return response.choices[0].message.content

    def generate_response(
        self,
        messages: List[Dict[str, str]],
        response_format=None,
        tools: Optional[List[Dict]] = None,
        tool_choice: str = "auto",
        **kwargs,
    ):
        """
        Generate a response based on the given messages using Zhipu AI.

        Args:
            messages (list): List of message dicts containing 'role' and 'content'.
            response_format (str or object, optional): Format of the response. Defaults to None.
            tools (list, optional): List of tools that the model can call. Defaults to None.
            tool_choice (str, optional): Tool choice method. Defaults to "auto".
            **kwargs: Additional Zhipu AI-specific parameters.

        Returns:
            str or dict: The generated response.
        """
        params = self._get_supported_params(messages=messages, **kwargs)

        params.update({
            "model": self.config.model,
            "messages": messages,
        })

        if response_format:
            params["response_format"] = response_format
        if tools:
            params["tools"] = tools
            params["tool_choice"] = tool_choice

        response = self.client.chat.completions.create(**params)
        parsed_response = self._parse_response(response, tools)

        response_callback = getattr(self.config, "response_callback", None)
        if response_callback:
            try:
                response_callback(self, response, params)
            except Exception as e:
                # Log error but don't propagate
                logging.error(f"Error due to callback: {e}")
                pass

        return parsed_response



"""
OceanBase storage module initialization
"""

from .oceanbase import OceanBaseVectorStore
from .oceanbase_graph import MemoryGraph
from .models import Base, create_memory_model

__all__ = [
    "OceanBaseVectorStore",
    "MemoryGraph",
    "Base",
    "create_memory_model",
]



from powermem.integrations.llm.config.openai import OpenAIConfig
from powermem.settings import settings_config


class OpenAIStructuredConfig(OpenAIConfig):
    """
    Configuration class for OpenAI Structured Output.
    Inherits all configuration from OpenAIConfig, only overrides metadata.
    """

    _provider_name = "openai_structured"
    _class_path = "powermem.integrations.llm.openai_structured.OpenAIStructuredLLM"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)



"""
PowerMem Scripts Package

Contains upgrade, downgrade, migration, and maintenance scripts.
"""

from script.scripts.upgrade_sparse_vector import upgrade_sparse_vector, downgrade_sparse_vector
from script.scripts.migrate_sparse_vector import migrate_sparse_vector

__all__ = [
    'upgrade_sparse_vector',
    'downgrade_sparse_vector',
    'migrate_sparse_vector',
]



from typing import Optional

from pydantic import AliasChoices, Field

from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.settings import settings_config


class OllamaConfig(BaseLLMConfig):
    """
    Configuration class for Ollama-specific parameters.
    Inherits from BaseLLMConfig and adds Ollama-specific settings.
    """

    _provider_name = "ollama"
    _class_path = "powermem.integrations.llm.ollama.OllamaLLM"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)

    # Ollama-specific fields
    ollama_base_url: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "ollama_base_url",
            "OLLAMA_LLM_BASE_URL",
        ),
        description="Ollama base URL"
    )



import importlib
from typing import Dict, Optional, Union

from powermem.integrations.llm.config.anthropic import AnthropicConfig
from powermem.integrations.llm.config.azure import AzureOpenAIConfig
from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.integrations.llm.config.deepseek import DeepSeekConfig
from powermem.integrations.llm.config.gemini import GeminiConfig
from powermem.integrations.llm.config.langchain import LangchainConfig
from powermem.integrations.llm.config.ollama import OllamaConfig
from powermem.integrations.llm.config.openai import OpenAIConfig
from powermem.integrations.llm.config.openai_structured import OpenAIStructuredConfig
from powermem.integrations.llm.config.qwen import QwenConfig
from powermem.integrations.llm.config.qwen_asr import QwenASRConfig
from powermem.integrations.llm.config.siliconflow import SiliconFlowConfig
from powermem.integrations.llm.config.vllm import VllmConfig
from powermem.integrations.llm.config.zai import ZaiConfig


def load_class(class_type):
    module_path, class_name = class_type.rsplit(".", 1)
    module = importlib.import_module(module_path)
    return getattr(module, class_name)


class LLMFactory:
    """
    Factory for creating LLM instances with appropriate configurations.
    Uses provider registration mechanism from BaseLLMConfig.
    """

    @classmethod
    def create(cls, provider_name: str, config: Optional[Union[BaseLLMConfig, Dict]] = None, **kwargs):
        """
        Create an LLM instance with the appropriate configuration.

        Args:
            provider_name (str): The provider name (e.g., 'openai', 'anthropic')
            config: Configuration object or dict. If None, will create default config from environment
            **kwargs: Additional configuration parameters (overrides)

        Returns:
            Configured LLM instance

        Raises:
            ValueError: If provider is not supported
        """
        # 1. Get class_path from registry
        class_path = BaseLLMConfig.get_provider_class_path(provider_name)
        if not class_path:
            raise ValueError(f"Unsupported Llm provider: {provider_name}")

        # 2. Get config_cls from registry
        config_cls = BaseLLMConfig.get_provider_config_cls(provider_name) or BaseLLMConfig

        # 3. Handle configuration
        if config is None:
            # Create default config from environment variables
            provider_settings = config_cls()
        elif isinstance(config, dict):
            # Create config from dict
            provider_settings = config_cls(**config)
        elif isinstance(config, BaseLLMConfig):
            # Use existing config as-is
            provider_settings = config
        else:
            raise TypeError(f"config must be BaseLLMConfig, dict, or None, got {type(config)}")

        # 4. Apply overrides (kwargs)
        if kwargs:
            provider_settings = provider_settings.model_copy(update=kwargs)

        # 5. Create LLM instance
        llm_class = load_class(class_path)
        return llm_class(provider_settings)

    @classmethod
    def register_provider(cls, name: str, class_path: str, config_class=None):
        """
        Register a new provider.

        Args:
            name (str): Provider name
            class_path (str): Full path to LLM class
            config_class: Configuration class for the provider (defaults to BaseLLMConfig)
        """
        if config_class is None:
            config_class = BaseLLMConfig

        # Register directly in BaseLLMConfig registry
        BaseLLMConfig._registry[name] = config_class
        BaseLLMConfig._class_paths[name] = class_path

    @classmethod
    def get_supported_providers(cls) -> list:
        """
        Get list of supported providers.

        Returns:
            list: List of supported provider names
        """
        return list(BaseLLMConfig._registry.keys())



"""
OceanBase utility functions

This module provides utility functions for checking OceanBase database information.
"""
import json
import logging
import re
from typing import Dict, Optional, List

try:
    from sqlalchemy import text
    from sqlalchemy.schema import CreateTable
    from pyobvector import FtsParser
    from pyobvector.schema import ObTable, VectorIndex, FtsIndex
    from pyobvector.client.index_param import IndexParams
    from pyobvector.client.fts_index_param import FtsIndexParam
    from pyobvector.client.partitions import ObPartition
except ImportError as e:
    raise ImportError(
        f"Required dependencies not found: {e}. Please install sqlalchemy and pyobvector."
    )

logger = logging.getLogger(__name__)


class OceanBaseUtil:
    """Utility class for OceanBase database checks and information retrieval."""

    @staticmethod
    def check_table_exists(obvector, table_name: str) -> bool:
        """
        Check if a table exists.

        Args:
            obvector: The ObVecClient instance.
            table_name: The name of the table.

        Returns:
            True if the table exists, False otherwise.
        """
        try:
            with obvector.engine.connect() as conn:
                result = conn.execute(text(
                    f"SELECT COUNT(*) FROM information_schema.TABLES "
                    f"WHERE TABLE_SCHEMA = DATABASE() "
                    f"AND TABLE_NAME = '{table_name}'"
                ))
                return result.scalar() > 0
        except Exception as e:
            logger.error(f"An error occurred while checking if table exists: {e}")
            return False

    @staticmethod
    def check_column_exists(obvector, table_name: str, column_name: str) -> bool:
        """
        Check if a column exists in a table.

        Args:
            obvector: The ObVecClient instance.
            table_name: The name of the table.
            column_name: The name of the column.

        Returns:
            True if the column exists, False otherwise.
        """
        try:
            with obvector.engine.connect() as conn:
                result = conn.execute(text(
                    f"SELECT COUNT(*) FROM information_schema.COLUMNS "
                    f"WHERE TABLE_SCHEMA = DATABASE() "
                    f"AND TABLE_NAME = '{table_name}' "
                    f"AND COLUMN_NAME = '{column_name}'"
                ))
                return result.scalar() > 0
        except Exception as e:
            logger.error(f"An error occurred while checking if column exists: {e}")
            return False

    @staticmethod
    def check_sparse_vector_column_exists(
        obvector, collection_name: str, sparse_vector_field: str
    ) -> bool:
        """
        Check if the sparse vector column exists.

        Args:
            obvector: The ObVecClient instance.
            collection_name: The name of the collection/table.
            sparse_vector_field: The name of the sparse vector field.

        Returns:
            True if the sparse vector column exists, False otherwise.
        """
        # Use the generic check_column_exists method
        return OceanBaseUtil.check_column_exists(obvector, collection_name, sparse_vector_field)

    @staticmethod
    def check_index_exists(obvector, table_name: str, index_name: str) -> bool:
        """
        Check if an index exists on a table (generic index check method).

        Args:
            obvector: The ObVecClient instance.
            table_name: The name of the table.
            index_name: The name of the index.

        Returns:
            True if the index exists, False otherwise.
        """
        try:
            with obvector.engine.connect() as conn:
                result = conn.execute(text(
                    f"SELECT COUNT(*) FROM information_schema.STATISTICS "
                    f"WHERE TABLE_SCHEMA = DATABASE() "
                    f"AND TABLE_NAME = '{table_name}' "
                    f"AND INDEX_NAME = '{index_name}'"
                ))
                return result.scalar() > 0
        except Exception as e:
            logger.error(f"An error occurred while checking if index exists: {e}")
            return False

    @staticmethod
    def check_sparse_vector_index_exists(obvector, collection_name: str) -> bool:
        """
        Check if the sparse vector index exists.

        Args:
            obvector: The ObVecClient instance.
            collection_name: The name of the collection/table.

        Returns:
            True if the sparse vector index exists, False otherwise.
        """
        # Use the generic check_index_exists method
        return OceanBaseUtil.check_index_exists(obvector, collection_name, "sparse_embedding_idx")

    @staticmethod
    def check_fulltext_index_exists(obvector, collection_name: str, fulltext_field: str) -> bool:
        """
        Check if the full-text index of the specified table exists.

        Args:
            obvector: The ObVecClient instance.
            collection_name: The name of the collection/table.
            fulltext_field: The name of the fulltext field.

        Returns:
            True if the full-text index exists, False otherwise.
        """
        try:
            with obvector.engine.connect() as conn:
                result = conn.execute(text(f"SHOW INDEX FROM {collection_name}"))
                indexes = result.fetchall()

                for index in indexes:
                    # Index [2] is the index name, index [4] is the column name, and index [10] is the index type
                    if len(index) > 10 and index[10] == 'FULLTEXT':
                        if fulltext_field in str(index[4]):
                            return True

                return False

        except Exception as e:
            logger.error(f"An error occurred while checking the full-text index: {e}")
            return False

    @staticmethod
    def is_seekdb(obvector) -> bool:
        """
        Check if the database is seekdb.

        Args:
            obvector: The ObVecClient instance.

        Returns:
            True if database is seekdb, False otherwise.
        """
        try:
            with obvector.engine.connect() as conn:
                # Try to get version information
                # OceanBase uses SELECT VERSION() or SHOW VARIABLES LIKE 'version'
                try:
                    result = conn.execute(text("SELECT VERSION()"))
                    version_str = result.fetchone()[0]
                except Exception:
                    # Fallback to SHOW VARIABLES
                    try:
                        result = conn.execute(text("SHOW VARIABLES LIKE 'version'"))
                        row = result.fetchone()
                        version_str = row[1] if row else ""
                    except Exception:
                        return False

                if not version_str:
                    return False

                version_str = str(version_str).strip().lower()
                return "seekdb" in version_str

        except Exception as e:
            logger.warning(f"Error checking if database is seekdb: {e}")
            return False

    @staticmethod
    def get_version_number(obvector) -> Optional[Dict[str, int]]:
        """
        Get the database version number.

        Args:
            obvector: The ObVecClient instance.

        Returns:
            Dictionary with keys "major", "minor", "patch" and int values, e.g., {"major": 4, "minor": 5, "patch": 0}.
            Returns None if version cannot be determined.
        """
        try:
            with obvector.engine.connect() as conn:
                # Try to get version information
                # OceanBase uses SELECT VERSION() or SHOW VARIABLES LIKE 'version'
                try:
                    result = conn.execute(text("SELECT VERSION()"))
                    version_str = result.fetchone()[0]
                except Exception:
                    # Fallback to SHOW VARIABLES
                    try:
                        result = conn.execute(text("SHOW VARIABLES LIKE 'version'"))
                        row = result.fetchone()
                        version_str = row[1] if row else ""
                    except Exception:
                        return None

                if not version_str:
                    return None

                version_str = str(version_str).strip()
                # Parse version string
                # For OceanBase, prioritize the actual OceanBase version (e.g., "5.7.25-OceanBase_CE-v4.3.5.5" -> 4.3.5)
                # First try to match OceanBase version pattern
                version_match = re.search(r'OceanBase[^v]*[vV]?(\d+)\.(\d+)\.(\d+)', version_str)
                if not version_match:
                    version_match = re.search(r'[vV]?(\d+)\.(\d+)\.(\d+)', version_str)
                
                if version_match:
                    major = int(version_match.group(1))
                    minor = int(version_match.group(2))
                    patch = int(version_match.group(3))
                    return {"major": major, "minor": minor, "patch": patch}
                else:
                    return None

        except Exception as e:
            logger.warning(f"Error getting database version number: {e}")
            return None

    @staticmethod
    def check_sparse_vector_version_support(obvector) -> bool:
        """
        Check if the database version supports sparse vector.

        Args:
            obvector: The ObVecClient instance.

        Returns:
            True if version is seekdb or OceanBase >= 4.5.0, False otherwise.
        """
        # Check if it's seekdb
        if OceanBaseUtil.is_seekdb(obvector):
            logger.info("Detected seekdb, sparse vector is supported")
            return True

        # Check if it's OceanBase and version >= 4.5.0
        version_dict = OceanBaseUtil.get_version_number(obvector)
        if version_dict is None:
            logger.warning("Could not determine database version, assuming sparse vector not supported")
            return False

        major = version_dict["major"]
        minor = version_dict["minor"]
        patch = version_dict["patch"]

        if major > 4 or (major == 4 and minor >= 5):
            logger.info(f"Detected OceanBase version {major}.{minor}.{patch}, sparse vector is supported")
            return True
        else:
            logger.warning(
                f"Detected OceanBase version {major}.{minor}.{patch}, "
                "sparse vector requires OceanBase >= 4.5.0"
            )
            return False

    @staticmethod
    def check_native_hybrid_version_support(obvector, table_name: str) -> bool:
        """
        Check if the database version and table type support native hybrid search (DBMS_HYBRID_SEARCH.SEARCH).

        Args:
            obvector: The ObVecClient instance.
            table_name: The name of the table.

        Returns:
            True if version is seekdb or OceanBase >= 4.4.1, and table is heap table or doesn't exist.
            False otherwise.
        """
        # Check if it's seekdb
        if OceanBaseUtil.is_seekdb(obvector):
            logger.info("Detected seekdb, native hybrid search is supported")
            # Also check if table is heap table (or doesn't exist)
            if not OceanBaseUtil.check_table_is_heap_or_not_exists(obvector, table_name):
                logger.warning(
                    f"Table '{table_name}' is not a heap table (ORGANIZATION HEAP). "
                    "Native hybrid search requires heap table."
                )
                return False
            return True

        # Check if it's OceanBase and version >= 4.4.1
        version_dict = OceanBaseUtil.get_version_number(obvector)
        if version_dict is None:
            logger.warning("Could not determine database version, assuming native hybrid search not supported")
            return False

        major = version_dict["major"]
        minor = version_dict["minor"]
        patch = version_dict["patch"]

        # Check version >= 4.4.1
        if major > 4 or (major == 4 and (minor > 4 or (minor == 4 and patch >= 1))):
            logger.info(f"Detected OceanBase version {major}.{minor}.{patch}, native hybrid search is supported")
            # Also check if table is heap table (or doesn't exist)
            if not OceanBaseUtil.check_table_is_heap_or_not_exists(obvector, table_name):
                logger.warning(
                    f"Table '{table_name}' is not a heap table (ORGANIZATION HEAP). "
                    "Native hybrid search requires heap table."
                )
                return False
            return True
        else:
            logger.warning(
                f"Detected OceanBase version {major}.{minor}.{patch}, "
                "native hybrid search requires OceanBase >= 4.4.1"
            )
            return False

    @staticmethod
    def check_table_is_heap_or_not_exists(obvector, table_name: str) -> bool:
        """
        Check if the table is a heap table (ORGANIZATION HEAP) or doesn't exist.

        DBMS_HYBRID_SEARCH.SEARCH requires heap table, not index-organized table.

        Args:
            obvector: The ObVecClient instance.
            table_name: The name of the table.

        Returns:
            True if table is heap table or doesn't exist, False if it's index-organized table.
        """
        try:
            with obvector.engine.connect() as conn:
                # Check if table exists first
                result = conn.execute(text(
                    f"SELECT COUNT(*) FROM information_schema.TABLES "
                    f"WHERE TABLE_SCHEMA = DATABASE() "
                    f"AND TABLE_NAME = '{table_name}'"
                ))
                if result.scalar() == 0:
                    # Table doesn't exist, will be created as heap table
                    logger.debug(f"Table '{table_name}' doesn't exist, will be created as heap table")
                    return True

                # Check table organization type using SHOW CREATE TABLE
                result = conn.execute(text(f"SHOW CREATE TABLE `{table_name}`"))
                row = result.fetchone()
                if row and len(row) >= 2:
                    create_statement = row[1]
                    # Check for ORGANIZATION keyword
                    if "ORGANIZATION INDEX" in create_statement.upper():
                        logger.debug(f"Table '{table_name}' is an index-organized table")
                        return False
                    elif "ORGANIZATION HEAP" in create_statement.upper():
                        logger.debug(f"Table '{table_name}' is a heap table")
                        return True
                    else:
                        # No ORGANIZATION keyword, default is index-organized in OceanBase
                        # when table has primary key
                        logger.debug(f"Table '{table_name}' has no explicit ORGANIZATION, assuming index-organized")
                        return False
                return False
        except Exception as e:
            logger.error(f"An error occurred while checking table organization type: {e}")
            return False

    @staticmethod
    def format_sparse_vector(sparse_dict: Dict[int, float]) -> str:
        """
        Format sparse vector dictionary to string format for SQL query.
        
        Args:
            sparse_dict: Dictionary with token_id as key and weight as value, e.g., {3: 0.3, 4: 0.4}
            
        Returns:
            Formatted string like '{3:0.3, 4:0.4}'
        """
        if not sparse_dict:
            return "{}"
        formatted = "{" + ", ".join(f"{k}:{v}" for k, v in sparse_dict.items()) + "}"
        return formatted

    @staticmethod
    def normalize(vector: List[float]) -> List[float]:
        """Normalize vector using L2 normalization."""
        import numpy as np
        arr = np.array(vector)
        norm = np.linalg.norm(arr)
        if norm == 0:
            return vector
        arr = arr / norm
        return arr.tolist()

    @staticmethod
    def get_fts_parser_enum(parser_name: str):
        """Convert parser name string to FtsParser enum.
        
        Args:
            parser_name: Parser name string (e.g., 'ik', 'ngram', 'ngram2', 'beng', 'space')
            
        Returns:
            FtsParser enum value
            
        Raises:
            ValueError: If parser name is not supported
        """
        parser_mapping = {
            'ik': FtsParser.IK,
            'ngram': FtsParser.NGRAM,
            'ngram2': FtsParser.NGRAM2,
            'beng': FtsParser.BASIC_ENGLISH,
            'space': None,
            'jieba': FtsParser.JIEBA,
        }
        
        parser_lower = parser_name.lower()
        if parser_lower not in parser_mapping:
            supported = ', '.join(parser_mapping.keys())
            raise ValueError(
                f"Unsupported fulltext parser: {parser_name}. "
                f"Supported parsers are: {supported}"
            )
        
        return parser_mapping[parser_lower]

    @staticmethod
    def parse_metadata(metadata_json):
        """
        Parse metadata from OceanBase.

        SQLAlchemy's JSON type automatically deserializes to dict, but this method
        handles backward compatibility with legacy string-serialized data.
        """
        if isinstance(metadata_json, dict):
            # SQLAlchemy JSON type returns dict directly (preferred path)
            return metadata_json
        elif isinstance(metadata_json, str):
            # Legacy compatibility: handle manually serialized strings
            try:
                # First attempt to parse
                metadata = json.loads(metadata_json)
                # Check if it's still a string (double encoded - legacy bug)
                if isinstance(metadata, str):
                    try:
                        # Second attempt to parse
                        metadata = json.loads(metadata)
                    except json.JSONDecodeError:
                        metadata = {}
                return metadata
            except json.JSONDecodeError:
                return {}
        else:
            return {}

    @staticmethod
    def check_sparse_vector_ready(obvector, collection_name: str, sparse_vector_field: str) -> bool:
        """
        Check if sparse vector support is ready (without creating anything).
        
        This method checks if sparse vector features are available:
        1. Database version supports sparse vector
        2. sparse_embedding column exists
        3. sparse_embedding_idx exists
        
        If any check fails, a warning is logged and False is returned.
        
        Args:
            obvector: The ObVecClient instance.
            collection_name: The name of the collection/table.
            sparse_vector_field: The name of the sparse vector field.
        
        Returns:
            bool: True if sparse vector is fully supported, False otherwise.
        """
        # Check if database version supports sparse vector
        if not OceanBaseUtil.check_sparse_vector_version_support(obvector):
            logger.warning(
                "Sparse vector support disabled: Database version does not support sparse vector. "
                "Sparse vector requires seekdb or OceanBase >= 4.5.0. "
                "Please upgrade your database to enable sparse vector support."
            )
            return False

        # Check if sparse_embedding column exists
        if not OceanBaseUtil.check_sparse_vector_column_exists(obvector, collection_name, sparse_vector_field):
            logger.warning(
                f"Sparse vector support disabled: Table '{collection_name}' does not have sparse_embedding column. "
                f"Please run the upgrade script to enable sparse vector support:\n"
                f"  from powermem import auto_config\n"
                f"  from script import ScriptManager\n"
                f"  config = auto_config()\n"
                f"  ScriptManager.run('upgrade-sparse-vector', config)"
            )
            return False

        # Check if sparse_embedding_idx index exists
        if not OceanBaseUtil.check_sparse_vector_index_exists(obvector, collection_name):
            logger.warning(
                f"Sparse vector support disabled: Table '{collection_name}' does not have sparse_embedding_idx index. "
                f"Please run the upgrade script to enable sparse vector support:\n"
                f"  from powermem import auto_config\n"
                f"  from script import ScriptManager\n"
                f"  config = auto_config()\n"
                f"  ScriptManager.run('upgrade-sparse-vector', config)"
            )
            return False
        
        logger.info(f"Sparse vector support validated successfully for table '{collection_name}'")
        return True

    @staticmethod
    def check_filters_all_in_columns(filters: Optional[Dict], model_class) -> bool:
        """
        Check if all filter fields are in standard table columns.

        This is used to determine if native hybrid search can be used.
        Native hybrid search doesn't support JSON path filtering well,
        so we only use it when all filters are on actual table columns.

        Args:
            filters: The filter conditions in mem0 format.
            model_class: SQLAlchemy ORM model class with __table__ attribute.

        Returns:
            True if all filter fields are in table columns, False otherwise.
        """
        if not filters:
            return True

        # Get column names from model_class
        try:
            table_columns = set(model_class.__table__.c.keys())
        except AttributeError:
            logger.warning("model_class does not have __table__ attribute, native hybrid search disabled")
            return False

        def check_filter_keys(filter_dict: Dict) -> bool:
            """Recursively check if all keys are in table columns."""
            for key, value in filter_dict.items():
                # Handle AND/OR logic
                if key in ["AND", "OR"]:
                    if isinstance(value, list):
                        for sub_filter in value:
                            if not check_filter_keys(sub_filter):
                                return False
                else:
                    # Check if this key is a table column
                    if key not in table_columns:
                        logger.debug(f"Filter field '{key}' not in table columns, native hybrid search disabled")
                        return False
            return True

        return check_filter_keys(filters)

    @staticmethod
    def convert_filters_to_native_format(
        filters: Optional[Dict],
        model_class,
        metadata_field: str = "metadata"
    ) -> List[Dict]:
        """
        Convert filter format to OceanBase native DBMS_HYBRID_SEARCH.SEARCH filter format.

        Follows the SEARCH API specification:
        - term: Exact match (strings, numbers, booleans)
        - range: Range queries (gte, gt, lte, lt)
        - match: Fuzzy match (full-text)
        - bool: Complex logic (must, should, must_not, filter)

        Args:
            filters: The filter conditions in mem0 format.
            model_class: SQLAlchemy ORM model class with __table__ attribute.
            metadata_field: Name of the metadata field (default: "metadata").

        Returns:
            List[Dict]: Filter conditions in OceanBase SEARCH native format.
        """
        if not filters:
            return []

        # Get column names from model_class
        try:
            table_columns = set(model_class.__table__.c.keys())
        except AttributeError:
            logger.warning("model_class does not have __table__ attribute")
            table_columns = set()

        def get_field_name(key: str) -> Optional[str]:
            """Get the field name for filter."""
            if key in table_columns:
                return key
            else:
                # Skip non-table fields for native hybrid search
                logger.debug(f"Skipping non-table field in native hybrid search: {key}")
                return None

        def process_single_filter(key: str, value) -> Optional[Dict]:
            """Process a single filter condition."""
            field_name = get_field_name(key)
            if field_name is None:
                return None

            # List values -> IN query -> bool.should with multiple term queries
            if isinstance(value, list):
                if not value:
                    return None
                return {
                    "bool": {
                        "should": [{"term": {field_name: v}} for v in value]
                    }
                }

            # Dict values -> May be range query or other operators
            if isinstance(value, dict):
                range_ops = {"gte", "gt", "lte", "lt"}
                if any(op in value for op in range_ops):
                    range_params = {op: value[op] for op in range_ops if op in value}
                    return {"range": {field_name: range_params}}

                if "eq" in value:
                    return {"term": {field_name: value["eq"]}}

                if "ne" in value:
                    return {"bool": {"must_not": [{"term": {field_name: value["ne"]}}]}}

                if "in" in value:
                    if not isinstance(value["in"], list) or not value["in"]:
                        return None
                    return {
                        "bool": {
                            "should": [{"term": {field_name: v}} for v in value["in"]]
                        }
                    }

                if "nin" in value:
                    if not isinstance(value["nin"], list) or not value["nin"]:
                        return None
                    return {
                        "bool": {
                            "must_not": [
                                {"bool": {"should": [{"term": {field_name: v}} for v in value["nin"]]}}
                            ]
                        }
                    }

                if "like" in value or "ilike" in value:
                    query_str = value.get("like") or value.get("ilike")
                    query_str = str(query_str).replace("%", "").replace("_", " ").strip()
                    if query_str:
                        return {"match": {field_name: {"query": query_str}}}
                    return None

            # None values -> Not supported, skip
            if value is None:
                logger.warning(f"NULL filter not supported in native search, skipping: {key}")
                return None

            # Simple values -> term query
            return {"term": {field_name: value}}

        def process_complex_filter(filters_dict: Dict) -> List[Dict]:
            """Process complex filters with AND/OR logic."""
            if "AND" in filters_dict:
                conditions = []
                for sub_filter in filters_dict["AND"]:
                    result = process_complex_filter(sub_filter)
                    if result:
                        conditions.extend(result)
                if conditions:
                    return [{"bool": {"filter": conditions}}]
                return []

            if "OR" in filters_dict:
                conditions = []
                for sub_filter in filters_dict["OR"]:
                    result = process_complex_filter(sub_filter)
                    if result:
                        conditions.extend(result)
                if conditions:
                    return [{"bool": {"should": conditions}}]
                return []

            results = []
            for k, v in filters_dict.items():
                if k not in ["AND", "OR"]:
                    result = process_single_filter(k, v)
                    if result:
                        results.append(result)
            return results

        return process_complex_filter(filters)

    @staticmethod
    def parse_native_hybrid_results(
        result_json_str: str,
        primary_field: str = "id",
        text_field: str = "document",
        metadata_field: str = "metadata"
    ) -> List[Dict]:
        """
        Parse the JSON results from OceanBase native DBMS_HYBRID_SEARCH.SEARCH.

        Args:
            result_json_str: JSON string returned from DBMS_HYBRID_SEARCH.SEARCH
            primary_field: Name of the primary key field
            text_field: Name of the text content field
            metadata_field: Name of the metadata field

        Returns:
            List[Dict]: List of parsed result dictionaries, each containing:
                - vector_id: Primary key value
                - text_content: Text content
                - score: Relevance score from _score field
                - user_id, agent_id, run_id, actor_id: Standard ID fields
                - hash, created_at, updated_at, category: Standard fields
                - metadata_json: Raw metadata JSON
        """
        try:
            result_data = json.loads(result_json_str)

            if not isinstance(result_data, list):
                logger.warning(f"Unexpected result format: {type(result_data)}, expected list")
                return []

            output_list = []

            for doc in result_data:
                vector_id = doc.get(primary_field)
                score = doc.get("_score", 0.0)

                if not vector_id:
                    logger.warning(f"Document missing primary key '{primary_field}', skipping")
                    continue

                output_list.append({
                    "vector_id": vector_id,
                    "text_content": doc.get(text_field, ""),
                    "score": score,
                    "user_id": doc.get("user_id", ""),
                    "agent_id": doc.get("agent_id", ""),
                    "run_id": doc.get("run_id", ""),
                    "actor_id": doc.get("actor_id", ""),
                    "hash": doc.get("hash", ""),
                    "created_at": doc.get("created_at", ""),
                    "updated_at": doc.get("updated_at", ""),
                    "category": doc.get("category", ""),
                    "metadata_json": doc.get(metadata_field, {}),
                })

            logger.debug(f"Parsed {len(output_list)} results from native hybrid search")
            return output_list

        except json.JSONDecodeError as e:
            logger.error(f"Failed to parse native hybrid search JSON results: {e}")
            return []
        except Exception as e:
            logger.error(f"Error parsing native hybrid search results: {e}")
            return []


"""
Base configuration for rerank models
"""
from typing import Any, ClassVar, Dict, Optional, Union

try:
    import httpx
except ImportError:
    httpx = None

from pydantic import Field
from pydantic_settings import BaseSettings

from powermem.settings import settings_config


class BaseRerankConfig(BaseSettings):
    """Base configuration for rerank models
    
    This class uses pydantic-settings to support automatic loading from environment variables.
    All rerank provider configurations should inherit from this base class.
    
    Environment Variables:
        RERANK_ENABLED: Whether to enable reranker (default: False)
        RERANK_MODEL: The rerank model to use
        RERANK_API_KEY: API key for the rerank service
        RERANK_API_BASE_URL: Base URL for the rerank API endpoint
        RERANK_TOP_N: Default number of top results to return
    """

    model_config = settings_config("RERANK_", extra="allow", env_file=None)

    # Class variables for provider registration
    _provider_name: ClassVar[Optional[str]] = None
    _class_path: ClassVar[Optional[str]] = None
    _registry: ClassVar[dict[str, type["BaseRerankConfig"]]] = {}
    _class_paths: ClassVar[dict[str, str]] = {}

    # Configuration fields
    enabled: bool = Field(
        default=False,
        description="Whether to enable reranker"
    )
    model: Optional[str] = Field(
        default=None,
        description="The rerank model identifier to use"
    )
    api_key: Optional[str] = Field(
        default=None,
        description="API key for the rerank provider"
    )
    api_base_url: Optional[str] = Field(
        default=None,
        description="Base URL for the rerank API endpoint"
    )
    top_n: Optional[int] = Field(
        default=None,
        description="Default number of top results to return (can be overridden at runtime)"
    )
    http_client_proxies: Optional[Union[Dict, str]] = Field(
        default=None,
        description="Proxy settings for HTTP client"
    )
    http_client: Optional[Any] = Field(  # httpx.Client type
        default=None,
        exclude=True,
        description="HTTP client instance"
    )

    @classmethod
    def _register_provider(cls) -> None:
        """Register provider in the global registry."""
        provider = getattr(cls, "_provider_name", None)
        class_path = getattr(cls, "_class_path", None)
        if provider:
            BaseRerankConfig._registry[provider] = cls
            if class_path:
                BaseRerankConfig._class_paths[provider] = class_path

    def __init_subclass__(cls, **kwargs) -> None:
        """Called when a class inherits from BaseRerankConfig."""
        super().__init_subclass__(**kwargs)
        cls._register_provider()

    @classmethod
    def __pydantic_init_subclass__(cls, **kwargs) -> None:
        """Called by Pydantic when a class inherits from BaseRerankConfig."""
        super().__pydantic_init_subclass__(**kwargs)
        cls._register_provider()

    @classmethod
    def get_provider_config_cls(cls, provider: str) -> Optional[type["BaseRerankConfig"]]:
        """Get the config class for a specific provider."""
        return cls._registry.get(provider)

    @classmethod
    def get_provider_class_path(cls, provider: str) -> Optional[str]:
        """Get the class path for a specific provider."""
        return cls._class_paths.get(provider)

    @classmethod
    def has_provider(cls, provider: str) -> bool:
        """Check if a provider is registered."""
        return provider in cls._registry

    def model_post_init(self, __context: Any) -> None:
        """Initialize http_client after model creation."""
        if self.http_client_proxies and not self.http_client and httpx:
            self.http_client = httpx.Client(proxies=self.http_client_proxies)

    def to_component_dict(self) -> Dict[str, Any]:
        """Convert config to component dict format matching RerankConfig structure.
        
        Returns:
            Dict matching RerankConfig schema with 'enabled', 'provider', 'config' fields
        """
        return {
            "enabled": self.enabled,
            "provider": self._provider_name,
            "config": self.model_dump(exclude_none=True)
        }




"""
Storage layer for memory management

This module provides the storage abstraction and implementations.
"""

from .base import VectorStoreBase
from .factory import VectorStoreFactory, GraphStoreFactory
from .config.base import BaseVectorStoreConfig, BaseGraphStoreConfig

__all__ = [
    "VectorStoreBase",
    "VectorStoreFactory",
    "GraphStoreFactory",
    "BaseVectorStoreConfig",
    "BaseGraphStoreConfig",
]



from typing import Any, Optional

from pydantic import Field

from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.settings import settings_config


class LangchainConfig(BaseLLMConfig):
    """
    Configuration class for Langchain LLM wrapper.
    Inherits from BaseLLMConfig.
    """

    _provider_name = "langchain"
    _class_path = "powermem.integrations.llm.langchain.LangchainLLM"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)

    # Langchain uses a model object instead of string
    model: Optional[Any] = Field(
        default=None,
        description="Langchain LLM model object"
    )



"""Query rewrite prompt templates"""

DEFAULT_QUERY_REWRITE_INSTRUCTIONS = """Use the user information to fill in any vague or ambiguous parts of the query.
Preserve the original intent of the query.
If the query is already clear and unambiguous, leave it unchanged."""

QUERY_REWRITE_TEMPLATE = """# Task
Rewrite the query by clarifying any ambiguous or underspecified references based on the provided user information, making the query more precise.

# User Information
{profile_content}

# Requirements
{custom_instructions}

# Output
Output only the rewritten query—do not add any explanations.

# Query
{query}
"""


def build_query_rewrite_prompt(
    profile_content: str,
    query: str,
    custom_instructions: str = None,
) -> str:
    """
    Build the query rewrite prompt with user profile and query.

    Args:
        profile_content: User profile text
        query: Original query string
        custom_instructions: Optional custom instructions to add to the prompt.
                            If None, uses DEFAULT_QUERY_REWRITE_INSTRUCTIONS.

    Returns:
        Complete prompt string for LLM
    """
    instructions = custom_instructions or DEFAULT_QUERY_REWRITE_INSTRUCTIONS
    return QUERY_REWRITE_TEMPLATE.format(
        profile_content=profile_content,
        query=query,
        custom_instructions=instructions,
    )



from typing import Any, Callable, Optional

from pydantic import AliasChoices, Field

from powermem.integrations.llm.config.base import BaseLLMConfig
from powermem.settings import settings_config


class AzureOpenAIConfig(BaseLLMConfig):
    """
    Configuration class for Azure OpenAI-specific parameters.
    Inherits from BaseLLMConfig and adds Azure OpenAI-specific settings.
    """

    _provider_name = "azure"
    _class_path = "powermem.integrations.llm.azure.AzureLLM"

    model_config = settings_config("LLM_", extra="forbid", env_file=None)

    # Override base fields with Azure-specific validation_alias
    api_key: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "api_key",
            "LLM_API_KEY",
            "AZURE_OPENAI_API_KEY",
            "AZURE_API_KEY",
        ),
        description="Azure OpenAI API key"
    )

    # Azure OpenAI-specific fields
    azure_endpoint: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "azure_endpoint",
            "AZURE_ENDPOINT",
            "AZURE_OPENAI_ENDPOINT",
        ),
        description="Azure OpenAI endpoint URL"
    )
    
    api_version: Optional[str] = Field(
        default="2025-01-01-preview",
        validation_alias=AliasChoices(
            "api_version",
            "AZURE_API_VERSION",
        ),
        description="Azure OpenAI API version"
    )
    
    azure_ad_token_provider: Optional[Callable[[], str]] = Field(
        default=None,
        exclude=True,
        description="Callable that returns an Azure AD token"
    )
    
    deployment_name: Optional[str] = Field(
        default=None,
        validation_alias=AliasChoices(
            "deployment_name",
            "AZURE_DEPLOYMENT",
        ),
        description="Azure OpenAI deployment name (alias for model)"
    )

    def model_post_init(self, __context: Any) -> None:
        """Initialize fields after model creation."""
        super().model_post_init(__context)
        # Use deployment_name if provided, otherwise use model
        if self.deployment_name:
            self.model = self.deployment_name
